#include "3dpaths.h"

#include	"application.h"			//	need some info for the viewport class

//	includes for OpenGL
#include <gl\gl.h>				//	header file for the opengl32 library
#include <gl\glu.h>
#include <gl\glaux.h>


/********************************************************************
*	implementation  of class cPath									*
********************************************************************/

//	this will draw the pattern...as 3d lines..
void cPath::Draw(void)
{
	glPushAttrib(GL_LIGHTING_BIT);		//	store lighting state
	glDisable(GL_LIGHTING);
	glPushAttrib(GL_COLOR_BUFFER_BIT);	//	store the color
	glBegin(GL_LINE_LOOP);
		glColor3f(0,1,1);
		for( int i=0; i<m_PointList.size(); i++ )
		{
			glVertex3f(m_PointList[i].x,m_PointList[i].y,m_PointList[i].z);
		}
	glEnd();
	glPopAttrib();					//	restore the color
	glPopAttrib();					//	restore the lighting state
}

//	this will generate a random path
void cPath::Randomize(int numpoints)
{
	m_PointList.clear();
	m_PointList.reserve(numpoints);
	for(int i=0;i<numpoints;i++)
		m_PointList.push_back(point3d(((rand()%10)-5)/4.0f,((rand()%10)-5)/4.0f,0));
}

//	loads the path from a file
void cPath::Load(const char * filename)
{
	ifstream infile;
	infile.open(filename);
	
	if(!infile)Application()->ReportError("Path File Not Found");

	//	first line contains number of points
	int NumPoints;
	infile >>	NumPoints;
	infile >>	m_NumSteps;
	
	if(NumPoints<3)
		Application()->ReportError("Paths must be 3 points or more");

	point3d Curr;
	for(int i=0;i<NumPoints;i++)
	{
		infile >>	Curr.x;
		infile >>	Curr.y;
		infile >>	Curr.z;
		m_PointList.push_back(Curr);
	}
}

/*-----------------------end of class cPath------------------------*/

/********************************************************************
*	implementation  of class cBezierPath							*
********************************************************************/

//	starts the iterations, reports error if the number of control points isn't valid
void cBezierPath::Start()
{
	if(m_PointList.size()%3)
		Application()->ReportError("Bezier Path control points must be a factor of 3!");
	m_CurIndex	=	0;
	m_CurLoc	=m_PointList[0];	
	NextSet();
}


//	sets up next bezier points set. take the first as the last in the previous list
void cBezierPath::NextSet()
{
	//	if the current index+3 is the same as the size, read the last 3 points and the first
	//	else, read 4 points 
	if(m_CurIndex+3==m_PointList.size())
	{
		m_Interpolator.Set(m_NumSteps,m_PointList[m_CurIndex],m_PointList[m_CurIndex+1],m_PointList[m_CurIndex+2],m_PointList[0]);
		m_CurIndex	=	0;
		m_bDone		=	true;
	}
	else
	{
		m_Interpolator.Set(m_NumSteps,m_PointList[m_CurIndex],m_PointList[m_CurIndex+1],m_PointList[m_CurIndex+2],m_PointList[m_CurIndex+3]);
		m_CurIndex	+=	3;
	}
	m_Interpolator.Start();
}

//	advances a step..
void cBezierPath::Next()
{
	if(m_Interpolator.Done())		
		NextSet();
	else
		m_Interpolator.Next();
	m_CurLoc	=	m_Interpolator.GetCurr();
}
/*-------------end of class cBezierPath implementation-------------*/

/********************************************************************
*	class cEaseOutPath implementation								*
********************************************************************/
//	start just set the current point
void cEaseOutPath::Start()
{
	m_CurLoc	=	m_PointList[0];	
	m_CurIndex	=	0;
}

//	iterate to next point in path
void cEaseOutPath::Next()
{
	m_Interpolator.Interpolate();

	if(m_Interpolator.Done())
	{
		if(m_CurIndex+1==m_PointList.size())
		{
			m_Interpolator.Setup(m_PointList[m_CurIndex],m_PointList[0],m_NumSteps);
			m_CurIndex	=	0;
			m_bDone		=	true;
		}
		else
		{
			m_Interpolator.Setup(m_PointList[m_CurIndex],m_PointList[m_CurIndex+1],m_NumSteps);
			m_CurIndex++;
		}
	}
	m_CurLoc	=	m_Interpolator.GetValue();
}
/*-------------end of class cEaseOutPath implementation------------*/


/********************************************************************
*	class cEaseInOut Implementation									*
********************************************************************/

//	start just set the current point
void cEaseInOutPath::Start()
{
	m_CurLoc	=	m_PointList[0];	
	m_CurIndex	=	0;
}

//	go to next point on path
void cEaseInOutPath::Next()
{
	m_Interpolator.Interpolate();

	if(m_Interpolator.Done())
	{
		if(m_CurIndex+1==m_PointList.size())
		{
			m_Interpolator.Setup(m_PointList[m_CurIndex],m_PointList[0],m_NumSteps);
			m_CurIndex	=	0;
			m_bDone		=	true;
		}
		else
		{
			m_Interpolator.Setup(m_PointList[m_CurIndex],m_PointList[m_CurIndex+1],m_NumSteps);
			m_CurIndex++;
		}
	}
	m_CurLoc	=	m_Interpolator.GetValue();
}
/*-----------end of class cEaseInOutPath implementation------------*/
/********************************************************************
*	class cPatternPath Implementation								*
********************************************************************/

//	start just set the current point
void cPatternPath::Start()
{
	m_CurLoc	=	m_PointList[0];	
	m_CurIndex	=	0;
}

//	go to next point on path
void cPatternPath::Next()
{
	m_Interpolator.Interpolate();

	if(m_Interpolator.Done())
	{
		if(m_CurIndex+1==m_PointList.size())
		{
			m_Interpolator.Setup(m_PointList[m_CurIndex],m_PointList[0],m_NumSteps);
			m_CurIndex	=	0;
			m_bDone		=	true;
		}
		else
		{
			m_Interpolator.Setup(m_PointList[m_CurIndex],m_PointList[m_CurIndex+1],m_NumSteps);
			m_CurIndex++;
		}
	}
	m_CurLoc	=	m_Interpolator.GetValue();
}
/*-----------end of class cPatternPath implementation---------------*/

