#ifndef _3DPATHS_H
#define _3DPATHS_H
/********************************************************************
*	3dpaths.h														*
*	by shady el mously												*
*	part of the flulib engin										*
*	this file is the declaration for all the path objects			*
*	which can be used to set the motions of anything				*
********************************************************************/
#include "3dmath.h"	//	for point 3d and the interpolators
/********************************************************************
*	class cPath														*
*	base class for 3d paths											*
*	basicaly have a pointslist and a next and getcuur methods		*
********************************************************************/
class cPath
{
protected:

	int					m_CurIndex;			//	which point are we usin now..
	point3d				m_CurLoc	;		//	Current Location in the path

public:

	vector<point3d>		m_PointList	;		//	List of control points for the path
	int					m_NumSteps	;		//	Number of steps (like the speed)
	bool				m_bDone		;		//	will be true when done

	cPath()	:	m_CurIndex(0),m_bDone(false)
	{	
		//	All done	
	}

	virtual ~cPath()
	{}


	//	adds a point to the path
	void AddPoint(point3d& P)
	{m_PointList.push_back(P);}

	//	adds a point to the path
	void AddPoint(float x,float y,float z)
	{m_PointList.push_back(point3d(x,y,z));}


	//	Externnaly get current location
	point3d& GetCurr(){return m_CurLoc;}

	//	pure virtual functions, sould be defined in children classes

	//	loads the path from a file
	virtual void Load(const char *);

	//	starts the iterations
	virtual void Start()=0;

	//	Next move, return false when we reach the end (in looping mode)
	virtual void Next()=0;

	//	this will draw the path..
	virtual void Draw(void);

	//	returns true when the end is reached
	virtual bool Done(){return m_bDone;}

	//	this will generate a random pattern..
	void Randomize(int numpoints);

};
/*----------------end of class cPath declaration-------------------*/


/********************************************************************
*	class bezier path												*
*	represents a smooth 3d path between a set of points				*
*	this path could be followed by lights, cameras, and 3d objects	*
*	paths fromt his type should have a number of points as factor	*
*	of 4															*
********************************************************************/

class cBezierPath : public cPath
{
private:

	cBezier				m_Interpolator;		//	a	Bezier curve to help making the bezier path

	//	takes the next 4 points into the iteration
	void NextSet();


public:

	//	constructor does nothing
	cBezierPath()
	{	m_PointList.clear();	}


	//	constructs the path from a text file
	cBezierPath(const char * FileName)
	{	Load(FileName);		Start();	}

	//	starts the iterations, reports error if the number of control points isn't valid
	void Start();

	//	advances a step..
	void Next();

	bool Done()
	{	
		return (m_Interpolator.Done()&&m_bDone);
	}
};
/*-----------end of class bezier path declaration------------------*/


/********************************************************************
*	class cEaseOutPath												*
*	a series of points that can be given as a motion plan			*
*	for any 3d object												*
*	the idea here is hat the motion start fast between each			*
*	consecutive points and slows down while approacing the 2nd		*
********************************************************************/
class cEaseOutPath : public cPath
{
private :

	cEaseOutDivideInterpolation<point3d> m_Interpolator;

public:

	//	constructs the path from a text file
	cEaseOutPath(const char * FileName)
	{	Load(FileName);		Start();	}


	//	call this to start the path
	void Start();

	//	this will jump to the next point with easy out interpolation
	void Next();
};
/*------------------end of class cEaseOutPath----------------------*/


/********************************************************************
*	class cEaseInOutPath											*
*	a series of points that can be given as a motion plan			*
*	for any 3d object												*
*	the idea here is that the motion start slow between each		*
*	2 points and goes fast then slow down again while				*
*	approcahing the next											*
********************************************************************/
class cEaseInOutPath : public cPath
{
private :

	cEaseInOutInterpolation<point3d> m_Interpolator;

public:

	//	constructs the path from a text file
	cEaseInOutPath(const char * FileName)
	{	Load(FileName);		Start();	}


	//	call this to start the path
	void Start();

	//	this will jump to the next point with easy out interpolation
	void Next();
};
/*------------------end of class cEaseOutPath----------------------*/


/********************************************************************
*	class cPatternPath												*
*	a series of points that can be given as a motion plan			*
*	for any 3d object												*
********************************************************************/
class cPatternPath : public cPath
{
private :

	cLinearInterpolation<point3d> m_Interpolator;

public:

	//	constructor
	cPatternPath()
	{}

		//	constructs the path from a text file
	cPatternPath(const char * FileName)
	{	Load(FileName);		Start();	}


	//	call this to start the path
	void Start();

	//	this will jump to the next point with easy out interpolation
	void Next();
};
/*------------------end of class cEaseOutPath----------------------*/

#endif // _3DPATHS_H