/****************************************************************
*	flulibUI.cpp												*
*	By Shady El Mously											*
*	part of the flulib class library							*
*	this file is the implementation file for user interface		*
*	objects like buttons, boxes, etc..							*
****************************************************************/
#include	"3dui.h"		//	the header implemented here
#include	"graphics.h"
#include	"framecontroller.h"




/****************************************************************
*	class GLFont implementation									*
****************************************************************/


//	create a font
void GLFont::Create(const char* typeface,int height,int weight,DWORD italic)
{
	Destroy();

	HFONT fontid;
	int	charset;

	m_ListBase	=	0;

	if(m_ListBase=glGenLists(256)==0)
		return;


	if(stricmp(typeface,"symbol")==0)
		charset=SYMBOL_CHARSET;
	else
		charset=ARABIC_CHARSET;


	fontid=CreateFont(height,0,0,0,weight,italic,FALSE,FALSE,charset,OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,DRAFT_QUALITY,DEFAULT_PITCH,typeface);

	SelectObject(Graphics()->m_hDC,fontid);

	wglUseFontBitmaps(Graphics()->m_hDC,0,256,m_ListBase);

	GetCharWidth(Graphics()->m_hDC,0,255,m_Widths);
	m_Height	=	height;

	DeleteObject(fontid);

}

//	destroy the font
void GLFont::Destroy()
{
	if(m_ListBase)
		glDeleteLists(m_ListBase,256);
}


//	put a string in the current raster position
void GLFont::Puts(const char * str)
{
	glPushAttrib(GL_LIST_BIT);
	glListBase(m_ListBase);
	glCallLists(strlen(str),GL_UNSIGNED_BYTE,str);
	glPopAttrib();

}

//	put a formatted string in the current raster position
void GLFont::Printf(int align,const char* format,...)
{
	va_list	ap;							//	argument pointer
	unsigned char s[1024],*ptr;			//	output string
	int	width	;						//	with in pixels

	//	make the formatted string
	va_start(ap,format);
	vsprintf((char*)s,format,ap);
	va_end(ap);

	//	compute its width in pixels
	for(ptr=s,width=0;*ptr;ptr++)
		width+=m_Widths[*ptr];

	//	if a negative was sent, align on center
	if(align<0)
		glBitmap(0,0,0,0,-width,0,NULL);
	else if(align==0)
		glBitmap(0,0,0,0,-width/2,0,NULL);

	//	now put the string
	Puts((char*)s);
}


/****************************************************************
*		implementation of the cursor class						*
*		Direct inpout is used here								*
****************************************************************/

//	construct with a bitmap as the cursor icon..
cCursor::cCursor(char * IconFileName):cBitmap(IconFileName)
{
	m_Xcontrol.SetParams(200,0.75f);
	m_Ycontrol.SetParams(200,0.75f);
	m_Loc.x		=	Application()->m_Width/2;
	m_Loc.y		=	Application()->m_Height/2;
}

//	processes the cursor movement from the direct input mouse object
//	i should ctach the couse state before calling this function
void cCursor::Draw()
{
	m_Xcontrol.Apply(Mouse()->m_DeltaX);
	m_Ycontrol.Apply(-Mouse()->m_DeltaY);

	m_Loc.x	+=	m_Xcontrol.Output() ;	
	m_Loc.y	+=	m_Ycontrol.Output()	;	

	//	update the global mouse info..
	Mouse()->m_Loc		=	m_Loc;

	//	this will restrict the cursor in the screen...
	if(m_Loc.x<0)m_Loc.x=0;
	else
		if(m_Loc.x>Application()->m_Width)
			m_Loc.x=Application()->m_Width;

	if(m_Loc.y<0)m_Loc.y=0;
	else
		if(m_Loc.y>Application()->m_Height)
			m_Loc.y=Application()->m_Height;

	cBitmap::Draw(m_Loc.x,m_Loc.y);
}

/*-------------end of class cursor implementation--------------*/

/****************************************************************
*	member functions implementation for							*
*	button class												*
****************************************************************/

// default constructor used whe inheriteng only...
void cButton::Init()
{
	m_Position.x		=	0;
	m_Position.y		=	0;
	m_Image				=	0;
	m_bIsUnderMouse		=	false;
	m_bWasUnderMouse	=	false;
	m_bIsHolded			=	false;
	m_Function			=	DoNothing;
}

// public constructor
cButton::cButton(char * label,char * bmpfilename)
{
	Init();
	SetImage(bmpfilename);
}

//	set using a bitmap image
void cButton::SetImage(char * filename)
{
	if(filename)
	m_Image=new cTexture(filename);
	m_Dimensions.x=m_Image->m_Width;
	m_Dimensions.y=m_Image->m_Height;
}

// is it clicked??
bool cButton::IsClicked()
{
	// the mouse button was re;lease after holding the button
	if (m_bIsHolded&&!Mouse()->m_bLButtonDown&&m_bIsUnderMouse)
	{
		m_bIsHolded	=	false;
		return		true;
	}
	else return false;
}

//	processing the button...
void cButton::Process()
{
	if(Mouse()->m_Loc.x>m_Position.x&&Mouse()->m_Loc.x<m_Position.x+m_Dimensions.width&&Mouse()->m_Loc.y>m_Position.y&&Mouse()->m_Loc.y<m_Position.y+m_Dimensions.height)
		m_bIsUnderMouse=true;
	else
		m_bIsUnderMouse=false;

	// is it holded ot not..
	//  release the hold to false must be in call to is clicked
	//  because it is like an event
	if(m_bIsUnderMouse&&Mouse()->m_bLButtonDown)
		m_bIsHolded=TRUE;

	if(m_Image)
		m_Image->Draw(m_Position.x,m_Position.y);

	if(m_bIsUnderMouse)
	{
		glBegin(GL_LINE_LOOP);
			glVertex3f(m_Position.x,m_Position.y,0);
			glVertex3f(m_Position.x+m_Dimensions.width-1,m_Position.y,0);
			glVertex3f(m_Position.x,m_Position.y+m_Dimensions.height-1,0);
			glVertex3f(m_Position.x,m_Position.y,0);
		glEnd();
	}
	m_bWasUnderMouse=m_bIsUnderMouse;
}
/*-------------End of class bcBtton implementation--------------*/


/****************************************************************
*	class c3DCursor implementation								*
****************************************************************/
//	draw the button according to its position and update it too
void c3DCursor::Draw()
{
	float dt=FrameController()->m_DeltaTime;
	controller.Apply(point3d(dt*Mouse()->m_DeltaX,dt*Mouse()->m_DeltaY,0));
	Translate(controller.Output().x/250,-controller.Output().y/250,0);
	cRigidBody::Draw();
	*((point3d*)&Mouse()->m_Xpos)=m_Bounder->m_Position;
}
/*-----------end of class c3DCursor implementation-------------*/




/****************************************************************
*	member functions implementation for							*
*	button3d class												*
****************************************************************/

//	this will process and draw the 3d button
void c3DButton::Draw()
{
/*	if(m_bIsUnderMouse)
	{
		RotateLocal(0,0,PI/25);
		m_Angle+=PI/25;
		
		if(m_Angle>(PI*2))
			m_Angle=m_Angle-(PI*2);
	}*/

	point3d CurPos	=	*(point3d*)&Mouse()->m_Xpos;

	if(m_Angle>0&&m_Angle<(PI*2)&&!m_bIsUnderMouse)
	{
		RotateLocal(0,0,PI/25);
		m_Angle+=PI/25;
	}

	//Mouse()->m_Loc.x
	if(m_bIsUnderMouse&&m_Bounder->m_Position.z-m_RestLoc.z<0.5f)
	{
		Translate(0,0,0.05f);
	}

	if(!m_bIsUnderMouse&&m_Bounder->m_Position.z>m_RestLoc.z)
	{
		Translate(0,0,-0.05f);
	}
	
	//	first draw the button..
	cRigidBody::Draw();


	if (m_Bounder->Below(CurPos))
		m_bIsUnderMouse=true;
	else
		m_bIsUnderMouse=false;

	// is it holded ot not..
	//  release the hold to false must be in call to is clicked
	//  because it is like an event
	if (m_bIsUnderMouse&&Mouse()->m_bLButtonDown)
		m_bIsHolded=TRUE;
	else
		m_bIsHolded=FALSE;


	m_bWasUnderMouse=m_bIsUnderMouse;
}
//			End of class button3d implementation
/*-------------------------------------------------------------*/



/****************************************************************
*	member functions implementation for							*
*	c3DMenu class												*
****************************************************************/

//	constructor creates a menu described in a text file
c3DMenu::c3DMenu(char* FileName) : m_LineWords(cString::arg_v)
{
	m_SceneNames.clear();
	m_SceneButtons.clear();


	//	open the file
	m_Fptr	=	fopen(FileName,"r+t");

	//	if no file exist.. report that
	if(!m_Fptr)
		Application()->ReportError("Menu File Not Found!");

	//	no exit button yet
	m_pExitButton=0;

	//	we return into this loop again when any new section begins
	bool firsttime=true;

	while(1)
	{
		//	break if the file ended
		if(feof(m_Fptr))
			break;
		// else, prepare a line
		if(firsttime)	PrepareLine();

		//	if this is a comment, skip..
		if(IsComment())
			continue;

		if(NewSection())
		{
			if(m_LineWords[0]=="menu")				ReadMenu();
			else
			if(m_LineWords[0]=="button")			ReadButton();
			else	//	this is something unknown
				Application()->ReportError("Unknown Section!");
		}
		firsttime	=	false;
	}
}

//	read some attributes realted to the menu as whole
void c3DMenu::ReadMenu()
{

}

//	read data related to a button...it is exactly as the 3d object in the scene
void c3DMenu::ReadButton()
{
	//	reading a new button... which is simple a 3d object
	//	we will read line after line
	//	until we find a line with just one string
	//	only then the function should leave

	c3DButton * pNewObject;	//	pointer to the object which will be created now
	pNewObject	=	new c3DButton;

	//	what can be read about the object is
	
	//	first : the object type (should come at first)
	//	which may be one of these :
	//	box , followed by 3 floats describing its dimensions
	//	sphere, followed by a single float which is its radius
	//	water	,	 for a water object, folowed by 2 floats and 2 integers
	//	terrain ,	followed by 3 floats (dimenstions and scale) and a file name 
	//	h3d , name of an h3d file
	//	o3d , name of an o3d file
	//	3ds , name of a 3ds file

	//	additional stuff like :
	//	texture : a texture file name for the object (string)
	//	position : initial position for the object (3 floats)
	//	orientation: initial orintatin angles for the objects (3 floats)

	//	also..
	//	transvel	:	3 floats (translation velocity)
	//	rotvel		:	3 floats (roation velocity)
	//	transacc	:	3 floats (translation acceleration)
	//	rotacc		:	3 floats (roation acceleration)
	
	while(1)
	{
		//	if end of file... leave
		if(feof(m_Fptr))			break;
		//	ok..prepare the line
		PrepareLine();
		//	skip comments
		if(IsComment())				continue;
		//	if this is a new section, break from here
		if(NewSection())			break;


		//	if that was the object type:
		if(cString::arg_v[0]=="type")
		{
			//	make a check on the 3rd string

			//	if this is a box
			if(m_LineWords[2]=="box")
				pNewObject->CreateBox(atof(cString::arg_v[3]),atof(cString::arg_v[4]),atof(cString::arg_v[5]));
			else	//	if it is a sphere
			if(m_LineWords[2]=="sphere")
				pNewObject->CreateSphere(atof(cString::arg_v[3]));
			else	//	if it is a disk
			if(m_LineWords[2]=="disk")
				pNewObject->CreateDisk(atof(cString::arg_v[3]),atof(cString::arg_v[4]));
			else	//	if it is a cylinder
			if(m_LineWords[2]=="cylinder")
				pNewObject->CreateCylinder(atof(cString::arg_v[3]),atof(cString::arg_v[4]),atof(cString::arg_v[5]));
			else	//	h3d files
			if(m_LineWords[2]=="h3d")
				pNewObject->CreateObject(cString::arg_v[3]);
			else	//	o3d files
			if(m_LineWords[2]=="o3d")
				pNewObject->CreateObject(cString::arg_v[3]);
			else	//	3ds files
			if(m_LineWords[2]=="3ds")
				pNewObject->CreateObject(cString::arg_v[3]);
			else	//	text object
			if(m_LineWords[2]=="text")
				pNewObject->CreateText(cString::arg_v[3]);
			else	//	heightfield terrain
			if(m_LineWords[2]=="terrain")
				pNewObject->CreateTerrain(1,1,cString::arg_v[3],atof(cString::arg_v[4]));
			else	//	water mesh
			if(m_LineWords[2]=="water")
				pNewObject->CreateWater(1,1,atoi(cString::arg_v[3]),atoi(cString::arg_v[4]));
			else
				Application()->ReportError("Unknown button type!");
		}
		else	//	else, the object name
		if(m_LineWords[0]=="name")
			pNewObject->m_Help=m_LineWords[2];
		else	//	else, if the texture name
		if(m_LineWords[0]=="texture")
			pNewObject->SetTexture(cString::arg_v[2]);
		else	//	else if the position
		if(m_LineWords[0]=="position")
			pNewObject->Translate(atof(cString::arg_v[2]),atof(cString::arg_v[3]),atof(cString::arg_v[4]));
		else	//	else if the orientation
		if(m_LineWords[0]=="orientation")
			pNewObject->RotateLocal(atof(m_LineWords[2])*PI/180,atof(m_LineWords[3])*PI/180,atof(m_LineWords[4])*PI/180);
		else//	else if the scale
		if(m_LineWords[0]=="scale")
			pNewObject->Scale(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	else if this is the translation velocity
		if(cString::arg_v[0]=="diffuse")
			pNewObject->m_Material.diffuse=color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	the specular color
		if(cString::arg_v[0]=="specular")
			pNewObject->m_Material.specular=color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	the ambient color
		if(cString::arg_v[0]=="ambient")
			pNewObject->m_Material.ambient=color3(atof(m_LineWords[2]),atof(m_LineWords[3]),atof(m_LineWords[4]));
		else//	the scene name that the button activates
		if(cString::arg_v[0]=="scenename")
		{
			m_SceneNames.push_back(m_LineWords[2]);
			//	add to the scene buttons
			m_SceneButtons.push_back(pNewObject);
		}
		else//	is this an exit button
		if(cString::arg_v[0]=="exit")
		{
			//if(!(atoi(m_LineWords[2])==0))
				m_pExitButton	=	pNewObject;
		}
		else//	error
			Application()->ReportError("Unknown button attribute!");
	}

	//	adjust stuff realted to the button for its interaction
	pNewObject->m_Angle			=	0;
	pNewObject->m_Function		=	DoNothing;
	pNewObject->m_RestLoc		=	pNewObject->m_Bounder->m_Position;

	//	add the new object to the buttons list
	push_back(pNewObject);
}


//	this will prepare a line of text, skipping any empty lines
void c3DMenu::PrepareLine()
{
	//	only leave here is the file has ended 
	//	only we found something in the line
	bool Done=false;

	while(!Done)
	{
		if(feof(m_Fptr))
			break;

		cString::ReadTokenLine(m_Fptr);

		if(m_LineWords.size())
			Done	=	true;
	}
}

//	comment detection, always call after prepare 
bool c3DMenu::IsComment()
{
	//	if the first charcater in the first is '#', return true
	//	this can be removed
	if(m_LineWords.size()==0)
		return true;

	if(((char*)m_LineWords[0])[0]=='#')
			return true;

	return false;
}

//	new section detection
bool c3DMenu::NewSection()
{
	//	new section means a single string which isnt a part of a comment
	//	i should always call this after making sure this isnt a comment
	if(m_LineWords.size()==1)
		return true;
	return false;
}

//	draw the whole menu...by iterating on all objects
void c3DMenu::Draw()
{
	for (int i=0;i<size();i++)
	{
		(*this)[i]->Draw();
	}
}
/*---------------End of class c3DMenu3d -----------------------*/


/****************************************************************
*	class c3dslide impelmentation								*
****************************************************************/
c3DSlide::c3DSlide(char* filename,int NumVerts)
{
	//	create the skin
	CreateWater(1,1,80,60);
	SetTexture(filename);
	Scale(4,3,1);
}
/*-----------end of class c3DSlide implementation--------------*/

/****************************************************************
*	class cMarquee implementation								*
*	this is the basic marquee class, just type each string alone*
*	in the lower middle of the screen							*
****************************************************************/


//	always construct with a text file..
cMarquee::cMarquee(char* FileName)
{
	//	make sure there are no loaded sentences
	m_SentencesList.clear();


	//	set a default color
	m_Color	=	0xFF0000;

	//	loading string...
	static char DefaultString[]="Loading Marquee..Please wait!";

	//	safely make the string point to this tatic string
	m_CurString	=	DefaultString;


	//	open the text file
	FILE* fptr=fopen(FileName,"r+t");
	if(!fptr) return;

	//	warning..any line with more than 200 charcaters can make problems here
	char TempStr[200];

	m_Finished	=	false;

	//	this will read the whole senetences in the file
	while(1)
	{
		if(feof(fptr))	break;

		cString::ReadLine(fptr,TempStr);
		if(strlen(TempStr)==0)continue;

		//	replace tabs by spaces
		for(int i=0;i<strlen(TempStr);i++)
			if(TempStr[i]=='\t')TempStr[i]=' ';

		m_SentencesList.push_back(TempStr);
	}
	fclose(fptr);
	m_CurSentenceNum	=	0;

	//	center on the y position..
	//	at the begining of the 4/5 o f the screen
	m_Yposition			=	Application()->m_Height*4/5;
}


//	this is the thread for the basic marquee...
//	take string by string and make some delay between delays based on the length of the string
void cMarquee::Run(void)
{
	int Counter=0;
	int Max=0;

	SIZE s;

	while(!m_Finished)
	{
		if(m_CurSentenceNum==m_SentencesList.size())			break;

		m_CurString	=	(char*)m_SentencesList[m_CurSentenceNum].c_str();

		GetTextExtentPoint32(Graphics()->m_hDC,m_CurString,strlen(m_CurString),&s);

		//	center the string on X
		m_Xposition	=	(Application()->m_Width-s.cx)>>1;

		//	reset the counter
		Counter=0;

		//	and set the max count we need
		Max=strlen(m_CurString)*5;

		//	make a delay
		while(!m_Finished)
		{
			Sleep(20);
			Counter++;
			if(Counter==Max)
				break;
		}

		//	go to next sentence
		m_CurSentenceNum++;
	}
	Kill();
}

//	write the text
void cMarquee::Draw()
{
	Graphics()->WriteText(m_Xposition,m_Yposition,m_Color,m_CurString);
}
/*---------------end of class cMarquee implementation--------------*/


/********************************************************************
*	class cScrollMarquee											*
*	By Shady El Mously												*
*	this class inherts from the basic marquee and make it scroll	*
*	the strings from right to left on screen						*
*	the magic behind this class is iside the double ended queue		*
********************************************************************/

cScrollMarquee::cScrollMarquee(char* FileName):cMarquee(FileName)
{

	m_Xposition	=	0;

	//	fill the visble text with spaces initially.. and terminate it with a o
	for(int i=0;i<Graphics()->m_NumCharsPerLine;i++)
		m_VisibleText.push_back(' ');

	m_VisibleText.push_back(0);


	//	add an extra space at the end of each sentence
	for(i=0;i<m_SentencesList.size();i++)
	{
		m_SentencesList[i]+=' ';
	}
	m_Color	=	0xFF0000;
}


//	the thread for  the scrolling text marquee
void cScrollMarquee::Run(void)
{
	while(!m_Finished)
	{
		if(m_CurSentenceNum==m_SentencesList.size())
		{
			m_Finished=true;
			return;
		}

		for(int i=0;(!m_Finished)&&i<strlen(m_SentencesList[m_CurSentenceNum].c_str());i++)
		{
			m_Xposition=Graphics()->m_FontWidth;

			for(int j=0;j<Graphics()->m_FontWidth;j++)
			{
				Sleep(25);
				if(m_Finished)
				return;
				m_Xposition--;
			}
			if(m_Finished)
				return;

			//	remove one charcater from the front
			m_VisibleText.pop_front();
			//	remove the last charcater
			m_VisibleText.pop_back();
			//	add a new charcater
			m_VisibleText.push_back(m_SentencesList[m_CurSentenceNum][i]);
			//	and add 0 to terminate the string
			m_VisibleText.push_back(0);

			//	adjust the pointer
			m_CurString	=	&m_VisibleText.front();
		}
		m_CurSentenceNum++;
	}
	Kill();
}

/*------------end of class cScrollMarquee implementation-----------*/


/********************************************************************
*	class cMatrixMarquee											*
*	implements the marquee witha thread that writes the strings		*
*	character by character on the screen							*
********************************************************************/

//	the matrix marquee thread
void cMatrixMarquee::Run()
{
	int Counter	=	0	;
	int	Len		=	0	;
	char* TempStr;
	SIZE s;

	//	the string to accumulate to..m_CurString should point to this always
	string	m_MatrixString;

	while(!m_Finished)
	{
		if(m_CurSentenceNum==m_SentencesList.size())
		{
			m_Finished=true;
			return;
		}

		//	point out to the string we want now
		TempStr	=	(char*)m_SentencesList[m_CurSentenceNum].c_str();

		//	get the string length
		Len	=	strlen(TempStr);

		//	get its exact width in pixels
		GetTextExtentPoint32(Graphics()->m_hDC,TempStr,Len,&s);

		//	center the string on X
		m_Xposition	=	(Application()->m_Width-s.cx)>>1;

		//	set the character counter to zero
		Counter=0;

		//	make the matrix string empty
		m_MatrixString=" ";

		//	make the pointer that will point to the text to be written point to the begining of the matrix string
		m_CurString	=	m_MatrixString.begin();

		//	stay here until the thread dies
		while(!m_Finished)
		{
			//	add an additional character
			m_MatrixString+=TempStr[Counter];

			//	the pointer should be always to the begining of the matrix string
			m_CurString	=	m_MatrixString.begin();

			if(m_pTypeSound)
				m_pTypeSound->Play();

			//	wait a moment
			Sleep(100);

			//	advance for the next character
			Counter++;
			
			//	if the string is done.. go out
			if(Counter==Len)
				break;
		}

		//	wait a little after the atring is here
		Len*=5;
		Counter	=	0;

		//	leave the string for a while in its place
		while(!m_Finished)
		{
			Sleep(20);
			Counter++;
			if(Counter==Len)
				break;
		}
		//	advance for the next sentence
		m_CurSentenceNum++;
	}
	Kill();
}
/*---------------end of class cMatrixMarquee implementation-----------------*/