#ifndef _APPLICATION_H
#define _APPLICATION_H
/****************************************************************
*	application.h												*
*	Main Application header for the flulib class library		*
****************************************************************/

#include "base.h"		//	essential for singletton and other useful classes
#include "../resource.h"			//	Dialog Resource 

/****************************************************************
*	class cApplication											*
*	this will be the application class							*
*	it is a singlettoin holding all the main items				*
*	which will be need everywhere in all subsystems				*
*	most mebers here are globals and accessed via macros		*
*	this class acts as a cummunication medium between			*
*	all other big singleton classes								*
*	to be used everywhere										*
****************************************************************/
class cApplication : public singleton<cApplication>
{
private:

	cWindow			* m_MainWindow;			//	pointer to main application window
	bool		m_FatalError;

public:

	//	Windows API Specific stuff
	MSG			m_Msg;					//	the main global generic message
	HMENU		m_Menu;					//	If i have a menu i would create it using this handle

	//	Global information about the application
	eLanguage	m_Language;				//	langauge of the application..
	bool		m_IsActive;				//	Boolean saying if the app is active or not(has focus)
	bool		m_IsDead;				//	if this is true, he application should terminate
	bool		m_KeyPressed;			//	keyboard state
	bool		m_ChangingMode;			//	Boolean to be true while switching from mode to mode
	bool		m_InThread;				//	are we inside a thread or not ?
	

	//	data required about the graphics subsystem
	int			m_FrameRate;			//	the frame rate to be update by the frame controller
	int			m_BitDepth;				//	current bit depth 
	int			m_Width;				//	width of app
	int			m_Height;				//	height of app
	bool		m_IsWindowed;			//	windowed or not
	bool		m_TripleBuffer;			//	using triple buffering
	HINSTANCE	m_Instance;				//	instance of the application

	ofstream	m_ReportFile;			//	file to report succeses
	

	//	settings functions
	bool		LoadSettings();			//	used to read data from a settings file, return false if file doesnt exists
	void		SetDefaultSettings();	//	sets default settings
	void		SaveSettings();			//	save settings to settings file

	//	the main function 
	void		main(void);

	//	this will return true when its time to terminate everything
	bool		ProcessMessages();

	//	this is the application constructor
	//	here lies all what should happen the application
	//	starts 
  	cApplication(HINSTANCE);

	//	destructor, all application subsystem should be shuuted down from here
	~cApplication(void);

	//	Get the handle to the window on wchich the graphics operate
	HWND	GetHwnd(){return m_MainWindow->GetHwnd();}
	//	this can be used to change the caption of the graphics window
	void	SetLabel(char *string){m_MainWindow->SetLabel(string);}

	//	this is the dialog handler for the application..
	static BOOL CALLBACK ConfigDialogProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam);

	//	a static vector containing all possible screen modes..
	static vector <sScreenMode> m_AvailableModes;

	//	this is a static Message handler to be use with DirectX Based Apps
	static HRESULT CALLBACK MessageHandler(HWND hwnd,UINT message,WPARAM wparam,LPARAM lparam);

	//	Error And Success reporting
	void ReportError(const char *ErrStr,...)		;		//	reports error in a file, quit safely with a message

	//	Get From The App The Surrent Time as a String
	char * CurTime();									//	A String with the currect time and date.. used everywhere

	//	in multilanguage apps.. this function will return the correct filename if required
	void GetFullFileName(char * FileName,char* FullFileName);

	//	this will popup a dialog for slecting a file for opening or saving..
	char* OpenNewFile(const char *wndTitle );
};

inline cApplication	*&	Application		(void){return cApplication::GetSingletonPtr();}
/*----------------end of class cApplication declaration---------------------*/


#endif //_APPLICATION_H