/********************************************************************
*	flulibsound.cpp													*
*	By Shady El Mously												*
*	this is the implementation for the direct sound stuff related	*
*	to the	library													*
*	the singleton class which is the sound subsystem				*
*	and a class for music and another one for a wave sound			*
********************************************************************/


//	library includes
#include	"application.h"					//	to access the application main window..
#include	"audio.h"					//	the header implemented here
#include	<direct.h>						//	for get_cwd

/********************************************************************
*		Implemetation functions for the sound subsystem class		*
*	Using direct sound and direct music								*
********************************************************************/

//	initialize directXAudio
cAudioSubSystem::cAudioSubSystem()	
{
	//	set all to NULLS
	m_Performance	=	NULL;
	m_Loader		=	NULL;
	m_Path			=	NULL;
	m_Buffer		=	NULL;
	m_Listener		=	NULL;

	char	pathStr[MAX_PATH];
	WCHAR	wcharStr[MAX_PATH];


	// initialize COM
	if (FAILED(CoInitialize(NULL)))
		Application()->ReportError("Failed to initialize COM");

	// create the loader object
	if(FAILED(CoCreateInstance(CLSID_DirectMusicLoader,NULL,CLSCTX_INPROC,IID_IDirectMusicLoader8,(void**)&m_Loader)))
		Application()->ReportError("Couldn't create the loader object");
	
	// create the performance object
	if (FAILED(CoCreateInstance(CLSID_DirectMusicPerformance,NULL,CLSCTX_INPROC,IID_IDirectMusicPerformance8,(void**)&m_Performance)))    
		Application()->ReportError("Couldn't create DirectMusic preformance object");
	

	//	initialize direct music and direct sound
	if(FAILED(m_Performance->InitAudio(	NULL,NULL,NULL,DMUS_APATH_DYNAMIC_STEREO,64,DMUS_AUDIOF_ALL,NULL)))
		Application()->ReportError("Couldn't initialize DirectXAudio");
	
	//	create an audio path
	if(FAILED(m_Performance->CreateStandardAudioPath(DMUS_APATH_DYNAMIC_3D,64,TRUE,&m_Path)))
		Application()->ReportError("Couldn't create standard 3d audio path");

	//	get 3D Buffer in the audio path
	if(FAILED(m_Path->GetObjectInPath(0,DMUS_PATH_BUFFER,0,GUID_NULL,0,IID_IDirectSound3DBuffer,(void**)&m_Buffer)))
		Application()->ReportError("Couldn't Get 3d buffer from audio path");

	//	get the 3d buffer parameters
	m_SoundBufferParams.dwSize=sizeof(DS3DBUFFER);
	m_Buffer->GetAllParameters(&m_SoundBufferParams);

	//	set the new 3d buffer parameters
	m_SoundBufferParams.dwMode	=	DS3DMODE_HEADRELATIVE;
	m_Buffer->SetAllParameters(&m_SoundBufferParams,DS3D_IMMEDIATE);

	//	get the listener from the path
	if(FAILED(m_Path->GetObjectInPath(0,DMUS_PATH_PRIMARY_BUFFER,0,GUID_NULL,0,IID_IDirectSound3DListener,(void**)&m_Listener)))
		Application()->ReportError("Couldt get listener object !");

	//	get the listener parameters
	m_ListenerParams.dwSize	=	sizeof(DS3DLISTENER);
	m_Listener->GetAllParameters(&m_ListenerParams);

	//	set position of the listener
	m_ListenerParams.vPosition.x	=	0.0f;
	m_ListenerParams.vPosition.y	=	0.0f;
	m_ListenerParams.vPosition.z	=	0.0f;
	m_Listener->SetAllParameters(&m_ListenerParams,DS3D_IMMEDIATE);

	//	retreive the current directory
	GetCurrentDirectory(MAX_PATH,pathStr);

	//	convert to unicode string
	MultiByteToWideChar(CP_ACP,0,pathStr,-1,wcharStr,MAX_PATH);

	//	set the search directory
	m_Loader->SetSearchDirectory(GUID_DirectMusicAllTypes,wcharStr,FALSE);

	//	success
	Application()->m_ReportFile<<"DirectSound subsystem successfully Initialized"<<endl;
} 
cAudioSubSystem::~cAudioSubSystem()										//	shutdown sound/music and report success
{
	// If there is any music playing, stop it. This is 
	// not really necessary, because the music will stop when
	// the instruments are unloaded or the performance is    
	// closed down.

	SAFE_RELEASE(m_Buffer);
	SAFE_RELEASE(m_Listener);
	SAFE_RELEASE(m_Path);
	

	if (m_Performance)
		m_Performance->Stop(NULL, NULL, 0, 0 ); 
	// CloseDown and Release the performance object.    
	if (m_Performance)
	{
		m_Performance->CloseDown();
		m_Performance->Release();     
	} // end if
		// Release the loader object.
	if (m_Loader)
		m_Loader->Release();     
	// Release COM
	CoUninitialize(); 

	//	report success
	Application()->m_ReportFile<<"DirectSound subsystem successfully ShutDown   "<<endl;
} 


//	setting some listener attributes
void cAudioSubSystem::SetListenerPosition(point3d& Loc)
{
	m_Listener->SetPosition(Loc.x,Loc.y,Loc.z,DS3D_IMMEDIATE);
}

void cAudioSubSystem::SetListenerOrientation(point3d& Fwd,point3d& Up)
{
	m_Listener->SetOrientation(Fwd.x,Fwd.y,Fwd.z,Up.x,Up.y,Up.z,DS3D_IMMEDIATE);
}
/*-----------end of class cAudioSubsystem implementation-----------*/



/********************************************************************
*	class cSound implementation										*
********************************************************************/

//	constructor, load simple sound files
cSound::cSound(char * filename)
{
	m_Segment		=	NULL;  
	WCHAR wfilename[_MAX_PATH]; 
	MULTI_TO_WIDE(wfilename, filename);
	
	if(FAILED(Audio()->GetLoader()->LoadObjectFromFile(CLSID_DirectMusicSegment,IID_IDirectMusicSegment8,wfilename,(void**)&m_Segment)))
		Application()->ReportError("Coudlnt create sound object");
	
	//	download the band
	m_Segment->Download(Audio()->GetPerformance());
}

//	destructor releases the segment if present
cSound::~cSound()
{
	m_Segment->SetParam(GUID_Unload, -1, 0, 0, (void*)Audio()->GetPerformance());
	// Release the segment and set to null
	m_Segment->Release(); 
	m_Segment  = NULL;
}

// a function to stop a played segment
void cSound::Stop()
{
	// play segment and force tracking of state variable
	Audio()->GetPerformance()->StopEx(m_Segment,0,0);
}

// play the sound on a  secondary channel
void cSound::Play()
{
	// play segment and force tracking of state variable
	Audio()->GetPerformance()->PlaySegmentEx(m_Segment,NULL,NULL,DMUS_SEGF_DEFAULT|DMUS_SEGF_SECONDARY,0,NULL,NULL,NULL);
}

//	use this to set the repeat time
void cSound::SetRepeat(int n)
{
	m_Segment->SetRepeats(n);
}


//	check if the music is playing
bool cSound::IsPlaying(void)
{
	if(Audio()->GetPerformance()->IsPlaying(m_Segment,NULL) == S_OK) 
		return true;
	return false;
}

/*----------------end of class cSound implementation---------------*/

/********************************************************************
*	class cMusic implementation										*
********************************************************************/

//	overloaded constructor, loads a midi file on initialization
cMusic::cMusic(char * filename)
{
	m_dmsegment  = NULL;  
	m_dmsegstate = NULL;  
	Load(filename);
}

//	destructor releases the music segment if present
cMusic::~cMusic()
{
	if(m_dmsegment)
	{
		m_dmsegment->SetParam(GUID_Unload, -1, 0, 0, (void*)Audio()->GetPerformance()); 
		m_dmsegment->Release(); 
		m_dmsegment  = NULL;
	}
	m_dmsegstate = NULL;
}

//	load a midi file into the music segment
void cMusic::Load(char *filename)
{
	// this function loads a midi segment
	DMUS_OBJECTDESC ObjDesc; 

	IDirectMusicSegment* pSegment = NULL;
	// convert filename to wide string
	WCHAR wfilename[_MAX_PATH]; 
	MULTI_TO_WIDE(wfilename, filename);
	
	// setup object description
	memset(&ObjDesc,0,sizeof(ObjDesc)); ObjDesc.dwSize=sizeof(ObjDesc); 
	ObjDesc.guidClass = CLSID_DirectMusicSegment;
	wcscpy(ObjDesc.wszFileName, wfilename );
	ObjDesc.dwValidData = DMUS_OBJ_CLASS | DMUS_OBJ_FILENAME;
	
	// load the object and query it for the IDirectMusicSegment interface
	// This is done in a single call to IDirectMusicLoader::GetObject
	// note that loading the object also initializes the tracks and does 
	// everything else necessary to get the MIDI data ready for playback.
	Audio()->GetLoader()->GetObject(&ObjDesc,IID_IDirectMusicSegment, (void**) &pSegment);
	pSegment->SetParam(GUID_StandardMIDIFile,-1, 0, 0, (void*)Audio()->GetPerformance());
	pSegment->SetParam(GUID_Download, -1, 0, 0, (void*)Audio()->GetPerformance());
	m_dmsegment		=	pSegment;
	m_dmsegstate	=	NULL;
	m_dmsegment->SetRepeats(DMUS_SEG_REPEAT_INFINITE);
	
}



// a function to stop a midi played segment
void cMusic::Stop()
{
	// stop a midi segment
	if (m_dmsegment)
		Audio()->GetPerformance()->Stop(m_dmsegment, NULL, 0, 0);

}

// a function to play the music
void cMusic::Play()
{
	// play segment and force tracking of state variable
	if (m_dmsegment)
	Audio()->GetPerformance()->PlaySegmentEx(m_dmsegment,NULL,NULL,0,0,NULL,NULL,NULL);
}

/*--------------end of class cMusic implementation-----------------*/




/********************************************************************
*	class c3DSound implementation									*
********************************************************************/

//	make an audio path for the sound
c3DSound::c3DSound(char* filename): cSound(filename)
{

}


//	play the 3d sound
void c3DSound::Play()
{
	// play segment and force tracking of state variable
	Audio()->GetPerformance()->PlaySegmentEx(m_Segment,NULL,NULL,DMUS_SEGF_DEFAULT|DMUS_SEGF_SECONDARY,0,NULL,NULL,Audio()->GetPath());
}

//	set the sound position
void c3DSound::SetPosition(float x,float y,float z)
{
	//point3d p;
	Audio()->GetMainBuff()->SetPosition(x,y,z,DS3D_IMMEDIATE );
	Audio()->GetMainBuff()->SetMinDistance(20,DS3D_IMMEDIATE );
}
/*------------------end of c3DSound implementation------------------*/
