/********************************************************************
*	flulibmisc.cpp													*
*	By Shady El Mously												*
*	this file is main file of flulib class library					*
*	here lies all the general code									*
*	and here lies the WinMain()										*
*	from which the execution starts									*
*	it contains generic classes for hadnling windows API stuff		*
********************************************************************/


//	all library includes
#include "base.h"
#include "application.h"


/********************************************************************
*	class cString implementation									*
********************************************************************/

vector<cString>  cString::arg_v;


//	split a string into a vector tokens
void cString::Split(char * command)
{
	cString newstr;		//	a temporary string to get into
	int state=0;		//	0=notinword, 1=inword
	arg_v.clear();		//	clear the strings vector
	for(int x=0;x<(int)strlen(command);x++)//loop on the command string
	{
		if(state==0&&isspace(command[x]))			continue;
		if(state==1&&!isspace(command[x]))			newstr+=command[x];
		else
		if(state==1&&isspace(command[x]))
		{
			arg_v.push_back(newstr);
			state=0;
		}
		else
			if(state==0&&!isspace(command[x]))
			{
				newstr="";
				newstr+=command[x];
				state=1;
			}
	}
	if(state==1)
	{
			arg_v.push_back(newstr);
	}
}

//	this global function will read a full line in a file and fill a string with 
//	it useful for scanning files
void cString::ReadLine(FILE * file,char * string)
{
	int i=0; // location in the string
	char ch;
	// if it is end of file.. go back
	if(feof(file))
	{
		string = NULL;
		return;
	}
	while(1)
	{	
		ch=fgetc(file);
		// terminate the string and go out of loop
		if (ch=='\n' || ch==EOF)
		{
			string[i]='\0';
			break;
		}
		else
		{
			string[i]=ch;
			i++;
		}
	}
}


// this read a line of tokens from within a text file previously opened..
void cString::ReadTokenLine(FILE* file)
{
	//	first make a large enough C string to read the line in..
	char * longstring;
	longstring=new char[256];
	//	read the lino into it..
	ReadLine(file,longstring);
	//	then split it...
	Split(longstring);
}
/*------------------end of class cString implementation------------*/

/********************************************************************
*	class thread implementation										*
********************************************************************/

//	Number Of Active Threads
int cThread::s_NumActiveThreads=0;


//	this will start the shell as a thread (independant) function
void cThread::Start(void)
{
	//	if it is active, never run it again
	if(!m_bInThread)
	{
		s_NumActiveThreads++;
		Application()->m_InThread=true;
		m_bInThread=true;
		m_ThreadHandle=CreateThread(NULL,0,ThreadConsole,(LPVOID)this,0,&m_ThreadID);
	}
}

//	this will stop the threaded module and arrange everything else
void cThread::StopThread()
{
	if(!m_bInThread) return;
	//	set m_thread to be false
	m_bInThread=false;
	//	decrement the static meber.. number of active threads
	s_NumActiveThreads--;
	//	if zero active threads, set application flag..
	if(s_NumActiveThreads==0)			Application()->m_InThread=false;
	//	this will assure the stability iof the thread...
	WaitForSingleObject(m_ThreadHandle,INFINITE);
	//	if it is..close the thread.
	CloseHandle(m_ThreadHandle);
}
/*---------------end of class thread implementation----------------*/

/********************************************************************
*	class Window implementation										*
*	By Shady El Mously												*
********************************************************************/

//	default constructor, will register a class
//	using the default message handler
//	and will be using the main_instance also
cWindow::cWindow()
{
	m_ClassName="WINCLASS";
	Register(DefaultWinProc);
}

//	overloaded constructor, which will be frequently used..
//	set a default class name of WINCLASS
//	u must send the hinstance for which the window will be created
//	also a message handler function may be sent here
cWindow::cWindow(LPSTR ClassName,WNDPROC WinProc)
{
	m_ClassName=ClassName;
	Register(WinProc);
}


//	This will create a window and shows it
//	i must send the width and height of the client rect required
//	and the window style and label get default value if
//	nothing was sent
void cWindow::Create(int width,int height,DWORD WinStyle,LPSTR label)
{
	//	Get a Centered Position
	int xpos=(GetSystemMetrics(SM_CXFULLSCREEN)-width)	>>	1;
	int ypos=(GetSystemMetrics(SM_CYFULLSCREEN)-height)	>>	1;
	m_Hwnd=CreateWindowEx(	NULL,
							m_ClassName,
							label,
							WinStyle,
							xpos<0 ? 0 : xpos,
							ypos<0 ? 0 : ypos,
							width,
							height,
							NULL,
							NULL,
							Application()->m_Instance,
							0	);
	RECT window_rect={xpos,ypos,xpos+width,ypos+height};
	//first adjust a rect struct to use
	AdjustWindowRect(&window_rect,
		GetWindowStyle(m_Hwnd),
		GetMenu(m_Hwnd)!=NULL
		);
	// now use it to correctly set the size of the window
	MoveWindow(m_Hwnd,
		xpos,ypos,
		window_rect.right-window_rect.left,
		window_rect.bottom-window_rect.top,TRUE);		
}



//	if no the message specific handler is sent to make the window class
HRESULT CALLBACK cWindow::DefaultWinProc(HWND hwnd,UINT message,WPARAM wparam,LPARAM lparam)
{
	PAINTSTRUCT ps;
	HDC hdc;
	switch(message)
	{
	case WM_DESTROY:
		//	only destroy if the application window is being destoryed..
		if(hwnd==Application()->GetHwnd())
		PostQuitMessage(0);
		return 0;
		break;

	case WM_PAINT:
		hdc=BeginPaint(hwnd,&ps);
		EndPaint(hwnd,&ps);
		return 0;
		break;
		
	default:
		break;
	}
	return DefWindowProc(hwnd,message,wparam,lparam);
}


//	fill a window class and register it...
void cWindow::Register(WNDPROC MessageHandler)
{
	WNDCLASSEX wc;
	wc.cbSize			=	sizeof(WNDCLASSEX);
	wc.cbClsExtra		=	0;
	wc.cbWndExtra		=	0;
	wc.hbrBackground	=	(HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.hCursor			=	LoadCursor(NULL,IDC_ARROW);
	wc.hIcon			=	LoadIcon(NULL,IDI_APPLICATION);
	wc.hIconSm			=	LoadIcon(NULL,IDI_APPLICATION);
	wc.hInstance		=	Application()->m_Instance;
	wc.lpfnWndProc		=	MessageHandler;
	wc.lpszClassName	=	m_ClassName;
	wc.lpszMenuName		=	NULL;
	wc.style			=	CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS;

	if(!RegisterClassEx(&wc))		Application()->ReportError("Could not Register windows class");
}

/*-----------------end of class window implementation-----------*/


