/****************************************************************
*	flulibcontrol.cpp											*
*	By Shady El Mously											*
*	part of the flulib class library							*
*	this is the implementation of the frame rate sensore and	*
*	controller singleton class									*
****************************************************************/

#include "framecontroller.h"		//	the header implemented here
#include <mmsystem.h>			//	for TimeGetTime()

/****************************************************************
*	member functions implementation for							*
*	FRAMECONTROL class											*
****************************************************************/


// this is a called once function which will set the timer necessary for synchronization
// the frame rate required is given here... by default it is 30
void cFrameController::SetFps(int fps)
{
	m_FrameRate=fps;
	// is there a performance counter ?? 
	if(QueryPerformanceFrequency((LARGE_INTEGER *)&perf_cnt))
	{
		// yes set timer and get starting time
		perf_flag=true;
		time_count=(DWORD)perf_cnt/fps;
		QueryPerformanceCounter((LARGE_INTEGER *)&next_time);
	}
	else
	{
		perf_cnt=1000;
		// no perfrmance counter.. use TimeGetTime
		next_time=timeGetTime();
		time_count=perf_cnt/fps;  // aprroximate
	}
}

//	this function will return true if the time to render a frame has occured
bool cFrameController::IsReady()
{
	if(perf_flag)		QueryPerformanceCounter((LARGE_INTEGER *)&cur_time);
	else				cur_time=timeGetTime();

	if(cur_time>next_time)
	{
		next_time=cur_time+time_count;
		return true;
	}
	
	return false;
}

//	adjusts the time settings of the frame controller
//	and calculate the average frame rate
void cFrameController::Update()
{
	static int frame=0;					
	//		store the new frame rate in the array
	static int timediff;
	timediff=(timeGetTime()-frametime);

	if(frame)
	m_DeltaTime	=	(float)timediff;
	if(timediff==0)
		timediff=1;
	fps_arr[frame]=1000/timediff;

	frame++;
	//	if the array is full.. calculate the average frame rate
	if(frame>NUM_FRAMES_AVERAGED-1)
	{
		int total=0;
		frame=0;
		for(int i=0;i<NUM_FRAMES_AVERAGED;i++)
			total+=fps_arr[i];
		total=total/i;
		Application()->m_FrameRate=total;
	}
	frametime=timeGetTime();
}

//	this should be used later for time dependant synchronization
double cFrameController::GetNewTimeDelta()
{
	LONGLONG currTime;
	double timeSpan;
	// Read the timer
	if( perf_flag )
	{
		QueryPerformanceCounter( (LARGE_INTEGER*) &currTime );
	}
	else
	{
		currTime = timeGetTime(); 
	}

	// Find the new time span by scaling the difference
	timeSpan = (currTime - lastTime) /perf_cnt;

	// Next time, the last time will be the current time. 
	lastTime = currTime; 

	// Return the new time
	return timeSpan;
}
//	end of class	FRAMECONTROL implementation
/*-------------------------------------------------------------*/
