/********************************************************************
*	graph.cpp														*
*	By Shady el Mously												*
*	implementing the graph search algorithm							*
*	this file contain the member functions implementation			*
*	for classes and structure to solve graph search problems		*
********************************************************************/
//#include "stdafx.h"		//	uncomment this for MFC apps 
#include "graph.h"		//	header implemented here
#include <math.h>		//	for sqrt()

/********************************************************************
*	class cSearchState implementation								*
********************************************************************/

//	this function will be used to check if the index 
//	sent came in any other parent node of
//	this node..
//	in other words, it will check if the state was 
//	previously rached up in the tree
bool cSearchState::IsChildOf(int i)
{
	cSearchState * JumpNode	=	this;
	//	until we reach the root node
	while(JumpNode)
	{
		//	if the jumpnode index is the smae as i.. return true
		if(JumpNode->m_CurNode==i)
			return true;
		//	else move up in the tree
		JumpNode=JumpNode->m_ParentState;
	}
	return false;
}
/*-----------end of class cSearchState implementation--------------*/


/********************************************************************
*	clas cGraph implementation										*
********************************************************************/
//	This function adda a node to the graph at the sent X,Y location
//	i should add a node to the vector of nodes, 
//	and expand the links matrix in both directions
void cGraph::AddNode(point3d Pos)
{
	//	add expand the links matrix by 1 in both directions
	BoolVect Dummy;
	Dummy.push_back(false);

	//	increase columns by one
	for(int i=0;i<m_Links.size();i++)
	{
		m_Links[i].push_back(0);
		Dummy.push_back(false);
	}

	//	increase rows by one
	m_Links.push_back(Dummy);

	//	add the node
	m_Nodes.push_back(Pos);
}

//	links 2 nodes..just mark with the links with true values indicating 
//	that a link exist here
bool cGraph::LinkNodes(int src,int dst)
{
	if(m_Links.size()==0)
		return false;

	if(src<0||dst<0||src>m_Links.size()-1||dst>m_Links.size()-1)
		return false;

	m_Links[src][dst]	=	true;
	m_Links[dst][src]	=	true;

	return true;

}

//	drop a node, remove it from the nodes vector, and shrink the links matrix
bool cGraph::DropNode(int index)
{
	if(index<0||index>m_Links.size()-1)
		return false;

	//	then get rid of the node itself 
	m_Nodes.erase(&m_Nodes[index]);

	//	drop a column..
	m_Links.erase(&m_Links[index]);
	// drop a row
	for(int i=0;i<m_Nodes.size();i++)
	{
		m_Links[i].erase(&m_Links[i][index]);
	}
	return true;
}

//	this function returns the floating point distance between 2 nodes..
float cGraph::DistanceBetween(int src,int dst)
{
	//return -1 if any node index is invalid
	if(!(Exist(src)&&Exist(dst)))
		return -1;

	//	return the straight line distance..
	return point3d::dist(m_Nodes[src],m_Nodes[dst]);
}
/*---------------end of class cGraph implementation----------------*/



/********************************************************************
*	class cSearchTree implementation								*
*	implementing all graph search algorithms on the class cGraph	*
********************************************************************/

//	breadth first search, search level by level in the tree...
cSearchState*	cSearchTree::BreadthFirst(int src,int dst)
{
	//	the src or destination is not valid in the map..return NULL;
	if(!(m_RoadMap.Exist(src)&&m_RoadMap.Exist(dst)))
		return NULL;

	//	a pointer to the node when we expand it..
	cSearchState*	NodeToExpand		;

	//	destroy the tree if existing 
	SAFE_DELETE(m_pRootNode);

	//	create the initial state
	m_pRootNode	=	new cSearchState;
	//	and set its node to be the source..
	m_pRootNode->m_CurNode		=	src;

	//	now the algorithm cam start..
	//	1-make a qeque cotaining the initial state
	deque<cSearchState*>	StatesQueue;
	StatesQueue.push_back(m_pRootNode);


	//	2-until the queue is empty or a path is found...
	while(!StatesQueue.empty())
	{
		//	2.aif the state in the front of the queue is the solution return it
		if(StatesQueue.front()->m_CurNode==dst)
			return StatesQueue.front();

		//	if the node in front wasnt the solution..
		//	take it out of the queue...to expand it...
		NodeToExpand	=	StatesQueue.front();
		StatesQueue.pop_front();

		//	now check from the links matrix
		//	if the state to be expaned leads to a new place..
		for(int i=0;i<m_RoadMap.m_Links.size();i++)
		{
			//	if a true is found..
			//	then there is a link to node number i
			if(m_RoadMap.m_Links[NodeToExpand->m_CurNode][i])
			{
				//	if node i wasnt previously passe by this path
				//	we can expand to a new state now..
				if(!NodeToExpand->IsChildOf(i))
				{
					//	create a new state under the node to expand
					NodeToExpand->m_ChildStates.push_back(new cSearchState);			//	create the new state
					NodeToExpand->m_ChildStates.back()->m_ParentState=NodeToExpand;		//	mark in which node is it
					NodeToExpand->m_ChildStates.back()->m_CurNode	=	i;				//	set its parent
					StatesQueue.push_back(NodeToExpand->m_ChildStates.back());			//	and add it to the back of the queue
				}
			}
		}
	}
	//	reaching here mean that no solution was found.. return null
	return NULL;
}

//	much like the breadth first is the depth first..
//	the only different is that we push new states on front not back...
cSearchState*	cSearchTree::DepthFirst(int src,int dst)
{
	//	if the src or destination is not valid in the map..return NULL;
	if(!(m_RoadMap.Exist(src)&&m_RoadMap.Exist(dst)))
		return NULL;

	//	destroy the tree if it exists
	SAFE_DELETE(m_pRootNode);

	//	create the initial state
	m_pRootNode				=	new cSearchState;
	m_pRootNode->m_CurNode	=	src;

	//	set initial state cost to zero..
	deque<cSearchState*>	StatesQueue;
	StatesQueue.push_back(m_pRootNode);

	cSearchState*	NodeToExpand;

	while(!StatesQueue.empty())
	{
		//	if the state in the front of the queue is the solution return it
		if(StatesQueue.front()->m_CurNode==dst)
			return StatesQueue.front();

		NodeToExpand	=	StatesQueue.front();

		StatesQueue.pop_front();

		//	now check from the links matrix
		//	if the state to be expaned leads to a new place..
		for(int i=0;i<m_RoadMap.m_Links.size();i++)
		{
			//	if a true is found..
			//	then there is a link to node number i
			if(m_RoadMap.m_Links[NodeToExpand->m_CurNode][i])
			{
				//	if node i wasnt previously passe by this path
				//	we can expand to a new state now..
				if(!NodeToExpand->IsChildOf(i))
				{
					//	create a new state under the node to expand
					NodeToExpand->m_ChildStates.push_back(new cSearchState);		//	create the new node
					NodeToExpand->m_ChildStates.back()->m_CurNode	=	i;			//	mark in which node is it
					NodeToExpand->m_ChildStates.back()->m_ParentState=NodeToExpand;	//	set its parent
					StatesQueue.push_front(NodeToExpand->m_ChildStates.back());		//	and add it to the front of the queue
				}
			}
		}
	}
	//	reaching here mean that no solution was found.. return null
	return NULL;
}



//	and now the Astar algorithm
cSearchState*	cSearchTree::AStar(int src,int dst)
{
	//	the src or destination is not valid in the map..return NULL;
	if(!(m_RoadMap.Exist(src)&&m_RoadMap.Exist(dst)))
		return NULL;

	cSearchState*	NodeToExpand;
	
	//	destroy the tree
	SAFE_DELETE(m_pRootNode);

	//	create the initial state
	m_pRootNode	=		new cSearchState;
	m_pRootNode->m_CurCost		=	0									;		//	cost accumulated so far in the initial state is zero
	m_pRootNode->m_CurNode		=	src									;		//	root node is in the src..
	m_pRootNode->m_CostToGoal	=	m_RoadMap.DistanceBetween(src,dst)	;		//	cost to goal is the straight line distance



	//	make a queue containing the initial state
	deque<cSearchState*>	StatesQueue;
	StatesQueue.push_back(m_pRootNode);

	//	until the quque is empty or a solution is found...
	while(!StatesQueue.empty())
	{
		//	if the state in the front of the queue is the solution return it
		if(StatesQueue.front()->m_CurNode==dst)
			return StatesQueue.front();

		//	if not.. take it out and for expansion
		NodeToExpand	=	StatesQueue.front();
		StatesQueue.pop_front();

		//	now check from the links matrix
		//	if the state to be expaned leads to a new place..
		for(int i=0;i<m_RoadMap.m_Links.size();i++)
		{
			//	if a true is found..
			//	then there is a link to node number i
			if(m_RoadMap.m_Links[NodeToExpand->m_CurNode][i])
			{
				//	if node i wasnt previously passe by this path
				//	we can expand to a new state now..
				if(!NodeToExpand->IsChildOf(i))
				{
					//	create a new state under the node to expand
					NodeToExpand->m_ChildStates.push_back(new cSearchState);

					
					//	set the node in which the state is in
					NodeToExpand->m_ChildStates.back()->m_CurNode		=	i;
					//	make the pointer point to the parent state
					NodeToExpand->m_ChildStates.back()->m_ParentState	=	NodeToExpand;
					//	calculate the heuristic value to the goal for the new state
					NodeToExpand->m_ChildStates.back()->m_CostToGoal	=	m_RoadMap.DistanceBetween(i,dst);
					//	and set the cost accumulated so far as the parent plus the distance passed..
					NodeToExpand->m_ChildStates.back()->m_CurCost		=	NodeToExpand->m_CurCost	+	m_RoadMap.DistanceBetween(i,NodeToExpand->m_CurNode);

					//	add it to the queue
					StatesQueue.push_back(NodeToExpand->m_ChildStates.back());
				}
			}
			//	now sort the whole queue accoring to the sum of the 
			//	cost to goal and the cost accumulated so far for each node
			sort(StatesQueue.begin(),StatesQueue.end(),cSearchState::SortRule);

			//	then use the unique algorithm to get rid of the states leading to same nodes
			//	sorting before calling unique will make sure that the least cost state 
			//	will be the only remaining state
			unique(StatesQueue.begin(),StatesQueue.end(),cSearchState::EqualityRule);
		}
	}
	//	reaching here mean that no solution was found.. return null
	return NULL;
}

//	returns a vector of integers which is the path found using BreadthFirst
vector<int>		cSearchTree::BreadthFirstPath(int src,int dst)
{
	vector<int> Path;		//	make an empty vector of integers contaning the path

	cSearchState* Solution	=	BreadthFirst(src,dst);
	//	if there is a solution.. build the path
	while(Solution)
	{
		Path.push_back(Solution->m_CurNode);		//	add each index to the path
		Solution=Solution->m_ParentState;
	}
	//	now i have the correct path but revesred, use revese algorithm will fix it..
	reverse(Path.begin(),Path.end());
	SAFE_DELETE(m_pRootNode);	
	return Path;
}

//	same thing using depth first
vector<int>		cSearchTree::DepthFirstPath(int src,int dst)
{
	vector<int> Path;
	cSearchState* Solution	=	DepthFirst(src,dst);
	//	if there is a solution.. build the path
	while(Solution)
	{
		Path.push_back(Solution->m_CurNode);	//	add each index to the path
		Solution=Solution->m_ParentState;
	}
	//	now i have the correct path but revesred, use revese algorithm will fix it..
	reverse(Path.begin(),Path.end());
	SAFE_DELETE(m_pRootNode);	
	return Path;
}

//	same using A*
vector<int>		cSearchTree::AStarPath(int src,int dst)
{
	vector<int> Path;
	cSearchState* Solution	=	BreadthFirst(src,dst);
	//	if there is a solution.. build the path
	while(Solution)
	{
		Path.push_back(Solution->m_CurNode);	//	add each index to the path
		Solution=Solution->m_ParentState;
	}
	//	now i have the correct path but revesred, use revese algorithm will fix it..
	reverse(Path.begin(),Path.end());
	SAFE_DELETE(m_pRootNode);	
	return Path;
}
/*---------------end of class cSearchTree implementation------------*/