#ifndef _INPUT_H
#define _INPUT_H

/****************************************************************
*	input.h														*
*	this is the main header for using direct input				*
*	everything is contained in a single class _INPUT			*
****************************************************************/

#include	"base.h"			//	for singletton class
#include	<dinput.h>				//	DirectInput Main Include

/****************************************************************
*	Class cInputSubSystem	definition							*
*	this class represents all the related stuff to direct		*
*	input from the keyboard and from the mouse					*
*****************************************************************/

class cInputSubSystem	:	public singleton<cInputSubSystem>
{
private:
	//	pointers to main Direct input stuff
	LPDIRECTINPUT8			m_lpdi		;		//pointer to main direct dfraw interface
	LPDIRECTINPUTDEVICE8	m_lpdikey	;		//pointer to keyboard device
	LPDIRECTINPUTDEVICE8	m_lpdimouse	;		//pointer to mouse device	

	//	the keyboard states
	UCHAR					m_keystate[256];
	//	this will initialize direct input
	void InitDI();
	//	use this to clean up direct input
	void EndDI()		{SAFE_RELEASE(m_lpdi);}
	//	use this to start using direct keyoard input
	void InitDKey();
	//	use this to finish using the direct keyboard
	void EndDKey()		{if(m_lpdikey)m_lpdikey->Unacquire();SAFE_RELEASE(m_lpdikey);}
	//	this will let you start using drect mouse
	void InitDMouse();
	/// to finish using the mouse
	void EndDMouse()	{if(m_lpdimouse)m_lpdimouse->Unacquire();SAFE_RELEASE(m_lpdimouse);}

	//	callback function for the devices enumeration
	static BOOL CALLBACK MyDIDevicesCallback(LPCDIDEVICEINSTANCE,LPVOID);


public:

	//	this will hold mouse state and position, global for external use
	DIMOUSESTATE		m_MouseState;
	//	default constructor set nulls
	cInputSubSystem();
	//	destructor will release all direct input stuff
	~cInputSubSystem();
	/// a function that initializes all direct input stuff
	void Init();
	//	this will return the desired key state
	//	u must use readkeyboard before it
	BOOL KeyDown(UCHAR key)	{return(m_keystate[key]&0x80);}
	//	this will catch all input keyboard and mouse
	//	i may try to make a thread which continuously catches input
	void Catch();


	void DetectDevices();

};


//	Global Access to DirectInput From Here..
inline cInputSubSystem	*&		Input()		{return cInputSubSystem::GetSingletonPtr();}
/*-------------------------------------------------------------*/
//				End of _INPUT definition

#endif// _FLULIBINPUT_H


