#ifndef _MESH3DS_H
#define _MESH3DS_H

/****************************************************************
*	mesh3ds.h													*
*	By Shady El Mously											*
*	part of the flulib 3d engine								*
*	this module is responsible on loading object from			*
*	3D Studio (.3ds) files										*
****************************************************************/


//	things that should be fixed inside the 3ds objects
//	1.until now i didnt do anything concerning the blended materials
//	2.i still dont use the matrices for each mesh
//	3.there is no hierarchical information
//	4.textures are restricted to 24-bit color bitmap files
//	5.scale and adjust functions should be rewritten again when the matruices are used

#include "3dtypes.h"
#include "meshes.h"

#define LOAD3DS_SUCCESFUL 1
#define ERR3DS_FILELOAD 2
#define ERR3DS_INVALIDVERSION 3


/****************************************************************
*	struct sMaterial3ds											*
*	derived from the normal sMaterial struct					*
*	and adding additional items specific to materials			*
*	imported from 3d studio										*
****************************************************************/

struct sMaterial3ds : public sMaterial
{
	bool HasTextureMap;	//tells us if mat has texture map
	bool HasMaterial;	//tells us if mat. comes with lighting info
	
	char mName[256];	//mesh name
	char tFileName[256];//texture file name(if any)

	DWORD dwColor;		//vertex color if we dont use lighting (not used yet)

	float Us, Vs;		//UV scale
	float uOff, vOff;	//UV offset
	float tRot;			//texture rotation angle
};

/*----------------end of struct sMaterial3ds---------------------*/



/****************************************************************
*	class cMesh3ds												*
*	derived from sIndexedMesh									*
*	represents a single mesh inside a 3ds file					*
****************************************************************/
class cMesh3ds : public sIndexedMesh
{
public:
	char mName[256];	//mesh name
	char matName[256];	//material name that "belongs: to this mesh(only one material is allowed per mesh object for simplicity)

	matrix4				Matrix;	//transformation matrix
	sMaterial3ds*		m_pMaterial;	//mesh's pointer to material

	DWORD dwNumVerts;			//number of verts in mesh
	DWORD dwNumFaces;			//number of faces in mesh
	cTexture*			lpTexture;	//texture map(if any) of mesh

	//	constructor initializes everything
	cMesh3ds();

	//	we dont need a destructor


	//	draw a single mesh virtual from base
	void Draw();

	//	adjust by adding some dummt texture coordiantes
	void Adjust();
};

/*----------------end of cMesh3ds declaration------------------*/


/****************************************************************
*	class c3dsObject											*
*	represnt an object loaded from a 3ds file					*
*	it read the data from a .3ds file and builds the object		*
*	and can be used to render it								*
****************************************************************/
struct s3dsMesh : public sMesh
{
public:
	//functions
	s3dsMesh();
	virtual ~s3dsMesh();

	int Load3DS(char *fileName);	//loads the 3DS file

	//variables/data structures
	bool bError;

	char FileName[MAX_PATH];
	DWORD dwTotalVerts;	//total verts in file
	DWORD dwTotalFaces;	//total faces in file

	int Version3DS;								//version number

	vector<sMaterial3ds>	m_MaterialList;			//	materials list
	vector<cMesh3ds>		m_MeshList;				//	meshes list
	vector<cTexture>		m_TextureList;			//	textures list

	void Draw();								//	draw the 3ds object
	void Scale(float amount);					//	scale it
	void Adjust();								//	adjust the 3ds object
	void WriteInfo();							//	will write some info about the object 
	void WriteInfo(char* filename);				//	this will write the full information in a file

private:


	//functions related to reading simple data types from the file
	//	those are essential because the data is in a strange format inside the 3ds files
	bool Read3DSHeader();	//gets file version and makes sure the file is compatible
	float PropBytetoFloat(BYTE val);	//changes 0-256 to 0.0f-1.0f
	unsigned char ReadChar(void);	//reads 8 bit chunk
	unsigned int ReadInt(void);	//reads 16 bit chunk
	unsigned long ReadLong(void);	//reads 32 bit chunk

	void Read3DSData();	//gather all info in 3ds file
	void ReadObjects();	//gather object info
	void ReadMaterials(DWORD Len);	//gather material info
	void ReadMesh(DWORD Len);	//gather mesh info
	void LinkMaterials();		//	link the materials into the meshes


	//variables/data structures
	DWORD Len;							//file posistion
	FILE *fle3DS;						//file stucture to get binary info for the 3DS file
};
/*-------------end of class c3dsObject declaration---------*/

#endif