#include "application.h"			//	needed to access some app specific functions..
#include "opengl.h"



//	Pointers to additional functions created from OpenGL Extensions
PFNGLMULTITEXCOORD2FARBPROC		glMultiTexCoord2fARBx		=	NULL;
PFNGLACTIVETEXTUREARBPROC		glActiveTextureARBx			=	NULL;
PFNGLCLIENTACTIVETEXTUREARBPROC	glClientActiveTextureARBx	=	NULL;
PFNGLLOCKARRAYSEXTPROC			glLockArraysx				=	NULL;
PFNGLUNLOCKARRAYSEXTPROC		glUnlockArraysx				=	NULL;

//	multitexture coordinate function
void	glMultiTexCoord2f(float S,float T)
{	glMultiTexCoord2fARBx(S,T,1);	}

//	set the active texture
void	glActiveTexture(int I)
{	glActiveTextureARBx(I);	}

//	set the client active texture
void	glClientActiveTexture(int I)
{	glClientActiveTextureARBx(I);	}

//	locking arrays..
void glLockArrays(int first,int count)
{
	glLockArraysx(first,count);
}

//	unlocking arrays
void glUnlockArrays()
{
	glUnlockArraysx();
}
/****************************************************************
*	class cOpenGLSubSystem implemenation						*
*	implementing Graphics with OpenGL							*
*	subsystem													*
****************************************************************/

//	this will initialize the Graphics System Using OpenGL
void cOpenGLSubSystem::InitGL()
{
	//	set the hdc and hrc to nulls before proceeding
	m_hRC	=	NULL;
	m_hDC	=	NULL;

	// Holds The Results After Searching For A Match
	GLuint	PixelFormat;						

	if (!Application()->m_IsWindowed)										// Attempt Fullscreen Mode?
	{
		DEVMODE dmScreenSettings;											// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));				// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);					// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= Application()->m_Width;			// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= Application()->m_Height;			// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= Application()->m_BitDepth;		// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;


		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
		{
			// If The Mode Fails, Offer Two Options.  Quit Or Run In A Window.
			Application()->ReportError("The Requested Fullscreen Mode Is Not Supported By\nYour Video Card.");
		}
	}

	if (!Application()->m_IsWindowed)								// Are We Still In Fullscreen Mode?
		MoveWindow(Application()->GetHwnd(),0,0,Application()->m_Width,Application()->m_Height,true);


	static	PIXELFORMATDESCRIPTOR pfd=					// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),					// Size Of This Pixel Format Descriptor
		1,								// Version Number
		PFD_DRAW_TO_WINDOW |			// Format Must Support Window
		PFD_SUPPORT_OPENGL |			// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,				// Must Support Double Buffering
		PFD_TYPE_RGBA,					// Request An RGBA Format
		Application()->m_BitDepth,		// Select Our Color Depth
		0, 0, 0, 0, 0, 0,				// Color Bits Ignored
		0,								// No Alpha Buffer
		0,								// Shift Bit Ignored
		0,								// No Accumulation Buffer
		0, 0, 0, 0,						// Accumulation Bits Ignored
		32,								// 16 Bit Z-Buffer (Depth Buffer)
		1,								// 16 bit Stencil Buffer
		0,								// No Auxiliary Buffer
		PFD_MAIN_PLANE,					// Main Drawing Layer
		0,								// Reserved
		0, 0, 0							// Layer Masks Ignored
	};

	// Did We Get A Device Context?
	if (!(m_hDC=GetDC(Application()->GetHwnd())))			
		Application()->ReportError("Can't Create A GL Device Context.");
	
	// Did Windows Find A Matching Pixel Format?
	if (!(PixelFormat=ChoosePixelFormat(m_hDC,&pfd)))
		Application()->ReportError("Can't Find A Suitable PixelFormat.");
	
	// Are We Able To Set The Pixel Format?
	if(!SetPixelFormat(m_hDC,PixelFormat,&pfd))
		Application()->ReportError("Can't Set The GL PixelFormat.");
	

	// Are We Able To Get A Rendering Context?
	if (!(m_hRC=wglCreateContext(m_hDC)))
		Application()->ReportError("Can't Create A GL Rendering Context.");

	// Try To Activate The Rendering Context
	if(!wglMakeCurrent(m_hDC,m_hRC))
		Application()->ReportError("Can't Activate The GL Rendering Context.");

	
	glShadeModel(GL_SMOOTH);								//	enable smooth shading...
	glClearColor(0.0f,0.0f,0.0f,0.0f);						//	cleared back color...is black..
	glClearDepth(1.0f);										//	set the depth buffer (much like the Z buffer)
	glEnable(GL_DEPTH_TEST);								//	Enable Depth Buffer
	glEnable(GL_CULL_FACE);									//	enable back face culling
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);		//	we want the best presective correction to be done..
	gluPerspective(45.0,1.3333,0.05f,1400);
	
	glFrontFace(GL_CCW);									//	the culling is clock wise
	ReSizeGLScene(Application()->m_Width,Application()->m_Height);		// Set Up Our Perspective 

	//	create the 3d font
	CreateOutLineFont("Andalus",15,0.2f);

	//	create the bitmap font
	CreateBitMapFont("System Fixed",20);

	//	enable vertex arrays for points normals and texture coordiantes
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	//	enable the default blending funcion
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);	//	set the blending function

	//	create the extensions
	CreateExtensions();
}

//	End OpenGL, Release everything realted to it..
void cOpenGLSubSystem::EndGL(void)
{

	//	if full screen.. restore the screen mode..
	if(!Application()->m_IsWindowed)
	{
		ChangeDisplaySettings(NULL,0);
		ShowCursor(TRUE);
	}

	//	release the hRC
	if (m_hRC)
	{
		if (!wglMakeCurrent(NULL,NULL))					// Are We Able To Release The DC And RC Contexts?
		{
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}

		if (!wglDeleteContext(m_hRC))					// Are We Able To Delete The RC?
		{
			Application()->ReportError("Release Rendering Context Failed.");
		}
		m_hRC=NULL;							// Set RC To NULL
	}

	if (m_hDC && !ReleaseDC(Application()->GetHwnd(),m_hDC))					// Are We Able To Release The DC
	{
		m_hDC=NULL;							// Set DC To NULL
	}


	//	deallocate all fonts stuff
	DestroyBitMapFont();
	DestroyOutLineFont();
	DeleteObject(m_GDIFont);
}

//	used to resize the gl window..
void cOpenGLSubSystem::ReSizeGLScene(int width,int height)
{
	if(height==0){height=1;}

	//	set the view port..
	glViewport(0,0,width,height);

	//	setting the projection
	glMatrixMode(GL_PROJECTION);

	//	first load identity..
	glLoadIdentity();

	//	calculate the aspect ratio of the window..
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,160.0f);
	
	//	set view matrix to identity
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

/********************************************************************
*	text display related functions									*
********************************************************************/

//	creates a bitmapfont with the required size and font
void cOpenGLSubSystem::CreateBitMapFont(char* fontname,int fontsize)
{
	m_FontHeight	=	fontsize;

	m_BitMapFontBase=glGenLists(96);

	if(stricmp(fontname,"symbol")==0)
	{
		m_GDIFont=CreateFont(fontsize,0,0,0,0,FALSE,FALSE,FALSE,
			SYMBOL_CHARSET,OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,FF_DONTCARE|FIXED_PITCH,fontname);

	}
	else
	{
		m_GDIFont=CreateFont(fontsize,0,0,0,0,FALSE,FALSE,FALSE,
			ARABIC_CHARSET,OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,FF_DONTCARE|FIXED_PITCH,fontname);
	}

	if(!m_GDIFont)
	{
		m_BitMapFontBase= 0;
		return;
	}
	SelectObject(m_hDC,m_GDIFont);
	wglUseFontBitmaps(m_hDC,32,96,m_BitMapFontBase);

	GetCharWidth(m_hDC,'A','A',&m_FontWidth);
	
	//	and now allocate the text buffer
	AllocateTextBuffer(Application()->m_Width/(m_FontWidth),Application()->m_Height/(m_FontHeight-1));

	//	using dc always requires a transparent background
	SetBkMode(m_hDC,TRANSPARENT);
}


//	create a 3d outlined font..
void cOpenGLSubSystem::CreateOutLineFont(char* fontname,int fontsize,float depth)
{
	HFONT hFont;

	m_OutLineFontBase=glGenLists(256);
	m_OutLineFontDepth=depth;

	if(stricmp(fontname,"symbol")==0)
	{
		hFont=CreateFont(fontsize,0,0,0,FW_BOLD,FALSE,FALSE,FALSE,
			SYMBOL_CHARSET,OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,FF_DONTCARE|FIXED_PITCH,fontname);
	}
	else
	{
		hFont=CreateFont(fontsize,0,0,0,FW_BOLD,FALSE,FALSE,FALSE,
			ANSI_CHARSET,OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,FF_DONTCARE|FIXED_PITCH,fontname);
	}
	if(!hFont)
	{
		m_OutLineFontBase= 0;
		return;
	}
	SelectObject(m_hDC,hFont);
	wglUseFontOutlines(m_hDC,0,255,m_OutLineFontBase,0.0f,depth,WGL_FONT_POLYGONS,m_GMF);

	DeleteObject(hFont);
}


//	will draw the text buffer charcaters, used in multithreading stuff..
void cOpenGLSubSystem::DrawTextBuffer()
{
	static char Cursor='_';
	static DWORD LastTime=GetTickCount();
	static bool	CursorVisible=true;


	InitDrawing2D();										//	initialize 2d drawing
	glPushAttrib(GL_ALL_ATTRIB_BITS);						//	save the state machine
	glListBase(m_BitMapFontBase-32);						//	set new list base
	glDisable(GL_DEPTH_TEST);								//	disable zbuffer test
	glColor3f(1.0f,1.0f,1.0f);								//	set the rasterization color to white

	// now draw all the screen text on the back buffer and flip it forward
	for(int y=0;y<m_NumLines;y++)
	{
		glRasterPos2f(0,(Application()->m_Height-((y+1)*(m_FontHeight-1))));
		glCallLists(strlen(m_ScreenText[y]),GL_UNSIGNED_BYTE,m_ScreenText[y]);
	}
	//	now draw the blinking cursor
	if(CursorVisible)
	{
		glRasterPos2f(m_CurCharPos*m_FontWidth,(Application()->m_Height-((m_CurLine+1)*(m_FontHeight-1))));
		glCallLists(1,GL_UNSIGNED_BYTE,&Cursor);
	}

	if(GetTickCount()-LastTime>100)
	{
		CursorVisible=!CursorVisible;
		LastTime	=	GetTickCount();
	}
	glPopAttrib();											//	restore state machine
	EndDrawing2D();

}

//	writes a 3d string in the current transformation place
void cOpenGLSubSystem::DrawText3D(char* str,float ypos)
{
	if(!str)		return;			//	if no string.. go back
	
	float length=0;					//	calculate here length of the string in floats
	int idx;						//	index when looping on the string

	for(idx=0;idx<strlen(str);idx++)//	calculate the length
		length+=m_GMF[str[idx]].gmfCellIncX;

	glPushAttrib(GL_ALL_ATTRIB_BITS);						//	save texturing state
	glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);	//	set automatic texture coordinates generation lineraly on S
	glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);	//	set automatic texture coordinates generation lineraly on T
	glEnable(GL_TEXTURE_GEN_S);								//	enable texture coordinates generation on S
	glEnable(GL_TEXTURE_GEN_T);								//	enable texture coordinates generation on T
	glTranslatef(-length/2.0f,ypos,m_OutLineFontDepth/2.0f);//	translate to center the string
	glPushAttrib(GL_LIST_BIT);								//	save display list state
	glListBase(m_OutLineFontBase);							//	set new list base
	glCallLists(strlen(str),GL_UNSIGNED_BYTE,str);			//	call lists that renders 3d text
	glPopAttrib();											//	restore display list bit
	glPopMatrix();											//	restore previous matrix
}


//	draws a string at the specified position on screen..	a 2D function
void cOpenGLSubSystem::DrawStr(char* str,int x,int y)
{
	glRasterPos2f(x,Application()->m_Height-y-m_FontHeight);//	set drawing position
	glPushAttrib(GL_LIST_BIT);								//	save display lists bit
	glListBase(m_BitMapFontBase-32);						//	set new list base
	glCallLists(strlen(str),GL_UNSIGNED_BYTE,str);			//	call lists that write text
	glPopAttrib();											//	restore display list bit
}

//	write a string, with a specified color, ignoring lighting
void	cOpenGLSubSystem::WriteText(int x,int y,int color,const char *str,...)
{
	va_list	ap;					//	argument pointer
	unsigned char s[1024];		//	output string

	//	make the formatted string
	va_start(ap,str);
	vsprintf((char*)s,str,ap);
	va_end(ap);

	glColor3ubv((const unsigned char*)&color);				//	set new color
	DrawStr((char*)s,x,y);									//	Draw the string
}

//	write the current screen mode
void	cOpenGLSubSystem::WriteMode	(void)
{
	WriteText(0,0,0xFFFFFF,"%dx%dx%d",Application()->m_Width,Application()->m_Height,Application()->m_BitDepth);
}

//	write the frame rate
void	cOpenGLSubSystem::WriteFPS	(void)
{													
	WriteText(Application()->m_Width-8*m_FontWidth,0,0xFFFFFF,"%d FPS",Application()->m_FrameRate);
}

void	cOpenGLSubSystem::WriteTime(void)
{
	char* timestr=Application()->CurTime() ;
	WriteText(Application()->m_Width-strlen(timestr)*m_FontWidth,Application()->m_Height-20,0xFFFFFF,timestr);
}


void cOpenGLSubSystem::WriteGDIText(char * str,int color,int x,int y)
{
	SetTextColor(m_hDC,color);
	TextOut(m_hDC,x,y,str,strlen(str));
}

//	uses the gdi font to write string ceneterd in the screen
void cOpenGLSubSystem::WriteTextCentered(char *sentence,int color,int ypos,int numchars)
{
	int width,height;

	width	=	Application()->m_Width;
	height	=	Application()->m_Height;

	SetTextColor(m_hDC,color);
	SetBkMode(m_hDC,TRANSPARENT);
	SelectObject(m_hDC,m_GDIFont);
	RECT rct;
	if(ypos==-1)
	{
		rct.top		=	(height>>1)-(height>>2);
		rct.bottom	=	(height>>1)+(height>>2);
	}
	else
	{
		rct.top		=	ypos;
		rct.bottom	=	ypos+(height>>2);
	}
	rct.right	=	width;
	rct.left	=	0;

	//	 the first line is not needed..
	DrawText(m_hDC,sentence,numchars,&rct,DT_CENTER|DT_VCENTER);
}

/*---------------end of text related functions------------------*/
//	call befire using any 2d grpahics functions
void	cOpenGLSubSystem::InitDrawing2D	(void)			
{	
	glDisable(GL_LIGHTING);					//	disable lighting
	glMatrixMode(GL_PROJECTION);			//	set matrix mode to projection
	glPushMatrix();							//	save current projection transform
	glLoadIdentity();						//	load identity to projection
	glOrtho(0.0f,Application()->m_Width-1,	//	set orthogonal projection
	0,Application()->m_Height-1,-1.0,1.0);	//	set orthogonal projection
	glMatrixMode(GL_MODELVIEW);				//	set matrix mode to model_view
	glPushMatrix();							//	save the current model view matrix
	glLoadIdentity();						//	load it as identity
	glColor3f(1.0f,1.0f,1.0f);				//	set the rasterization color to white
}

//	call when drawing in 2d has ended
void	cOpenGLSubSystem::EndDrawing2D	(void)		
{
	WriteText(0,Application()->m_Height-30,0xFFFFFF,"Powered by FLULIB 3D engine");
	glMatrixMode(GL_PROJECTION);			//	set matrix mode to projection
	glPopMatrix();							//	restore the previous projection transform
	glMatrixMode(GL_MODELVIEW);				//	set matrix mode to model view
	glPopMatrix();							//	restore the previous model view transform
	glEnable(GL_LIGHTING);					//	disable lighting
}


//	creation go the multitexturing functions
void cOpenGLSubSystem::CreateExtensions()
{
	char*	 ExtString=(char*)glGetString(GL_EXTENSIONS);	//	Get the extensions of the current implementation
	cString::Split(ExtString);								//	aplit it
	for(int i=0;i<cString::arg_v.size();i++)				//	find out if it contains the multitexturing support
	{
		if(cString::arg_v[i]=="GL_ARB_multitexture")
			m_bCanMultiTexture	=	true;
		if(cString::arg_v[i]=="GL_EXT_compiled_vertex_array")
			m_bCanLockArrays	=	true;
		if(cString::arg_v[i]=="GL_EXT_texture_env_combine")
			m_bSupportTexEnvCombine = true;
		if(cString::arg_v[i]=="GL_EXT_texgen_reflection"||cString::arg_v[i]=="GL_NV_texgen_reflection")
			m_bSupportReflectionMap = true;
	}

	//	now if multitextring is supported
	//	get the functions addresses and assign them
	if(m_bCanMultiTexture)
	{
		glMultiTexCoord2fARBx		=	(PFNGLMULTITEXCOORD2FARBPROC)wglGetProcAddress("glMultiTexCoord2fARB");
		glActiveTextureARBx			=	(PFNGLACTIVETEXTUREARBPROC)wglGetProcAddress("glActiveTextureARB");
		glClientActiveTextureARBx	=	(PFNGLCLIENTACTIVETEXTUREARBPROC)wglGetProcAddress("glClientActiveTextureARB");
		glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &m_MaxTextureUnits);		
	}

	if(m_bCanLockArrays)
	{
		glLockArraysx		=(PFNGLLOCKARRAYSEXTPROC)wglGetProcAddress("glLockArraysEXT");
		glUnlockArraysx		=(PFNGLUNLOCKARRAYSEXTPROC)wglGetProcAddress("glUnlockArraysEXT");
	}

	//	support for reflection mapping
	if(m_bSupportReflectionMap)
	{
		if(m_bCanMultiTexture)
		{
			glActiveTexture(GL_TEXTURE0_ARB);
			glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);

			glActiveTexture(GL_TEXTURE1_ARB);
			glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);

			glActiveTexture(GL_TEXTURE0_ARB);
		}
		else
		{
			glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
			glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_NV);
		}
	}
}



//	this function will save a screen shot of the front buffer in a bitmap file
void	cOpenGLSubSystem::SaveScreen(char* filename)
{
	FILE*	fptr						;	//	file pointer
	BITMAPFILEHEADER	bitmapFileHeader;	//	bitmap file header
	BITMAPINFOHEADER	bitmapInfoHeader;	//	bitmap info header
	int					i				;	//	counter
	unsigned	char	tempRGB			;	//	used for swapping

	//	open file for writing binary mode
	fptr=fopen(filename,"wb");
	if(!fptr)
		return;

	//	read the whole screen now..
	unsigned char *	ImageData;
	ImageData	=	(unsigned char*)malloc(Application()->m_Width*Application()->m_Height*3);
	glReadPixels(0,0,Application()->m_Width,Application()->m_Height,GL_RGB,GL_UNSIGNED_BYTE,ImageData);
	

	//	define the bitmap file header
	bitmapFileHeader.bfSize			=	sizeof(BITMAPFILEHEADER)							;	
	bitmapFileHeader.bfType			=	0x4D42												;
	bitmapFileHeader.bfReserved1	=	0													;
	bitmapFileHeader.bfReserved2	=	0													;
	bitmapFileHeader.bfOffBits		=	sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)	;


	//	define the bitmap information header
	bitmapInfoHeader.biSize			=	sizeof(BITMAPINFOHEADER)							;
	bitmapInfoHeader.biPlanes		=	1													;
	bitmapInfoHeader.biBitCount		=	24													;
	bitmapInfoHeader.biCompression	=	BI_RGB												;
	bitmapInfoHeader.biSizeImage	=	Application()->m_Width*Application()->m_Height*3	;
	bitmapInfoHeader.biXPelsPerMeter=	0													;
	bitmapInfoHeader.biYPelsPerMeter=	0													;
	bitmapInfoHeader.biClrUsed		=	0													;
	bitmapInfoHeader.biClrImportant	=	0													;
	bitmapInfoHeader.biWidth		=	Application()->m_Width								;
	bitmapInfoHeader.biHeight		=	Application()->m_Height								;

	//	switch the image data from RGB to BGR
	for(i=0;i<bitmapInfoHeader.biSizeImage;i+=3)
	{
		tempRGB	=	ImageData[i]			;
		ImageData[i]	=	ImageData[i+2]	;
		ImageData[i+2]	=	tempRGB			;
	}

	
	//	write the bitmap file header
	fwrite(&bitmapFileHeader,1,sizeof(BITMAPFILEHEADER),fptr);

	//	write the bitmap info  header
	fwrite(&bitmapInfoHeader,1,sizeof(BITMAPINFOHEADER),fptr);

	//	write the image data
	fwrite(ImageData,1,bitmapInfoHeader.biSizeImage,fptr);

	//	free the allocated image data after saving
	free(ImageData);
}

/*-------------end of class cOpenGLSubSystem implementation----------*/

