/********************************************************************
*	flulibphysics.cpp												*
*	By Shady El Mously												*
*	Part of the flulib class library								*
*	this is the code implementaiton of all the classes of the		*
*	physics layer													*
********************************************************************/

#include "physics.h"
#include "framecontroller.h"		//	for timing


/********************************************************************
*	class cBounder implementation									*
*	By Shady el Mously												*
********************************************************************/
//	construct from a rigid body its bounds
//	make the point3d pointer point exactly to the location of the body..inside its world matrix
cBounder::cBounder(cRigidBody* pBody): m_Position(*(point3d*)&pBody->m_WorldMatrix._41)
{
	//	All done !
}

/*---------------end of class cBounder implementation--------------*/


/********************************************************************
*	class cBounderSphere implementation								*
********************************************************************/
//	check the existence of a vertex ins ide the bounding sphere
bool cBoundSphere::CheckCollision(point3d& p)
{
	//	calculate the distance between the sent point and the center of the bouding sphere
	float Dist=(p-m_Position).mag();
	if(Dist<=m_Radius)
		return true;
	return false;
}


//	same text but only using x,y coordiantes, this is used for detecting cursor
bool cBoundSphere::Below(point3d& CursorPos)
{
	float dx=CursorPos.x-m_Position.x;
	float dy=CursorPos.y-m_Position.y;
	float Dist=sqrt(dx*dx+dy*dy);

	if(Dist<=m_Radius)
		return true;
	return false;
}

//	scale the sphere
void cBoundSphere::Scale(point3d& Scale)
{
	float max;
	max=Scale.x;
	if(Scale.y>max)
		max=Scale.y;
	if(Scale.z>max)
		max=Scale.z;
	m_Radius*=max;
}
/*--------------end of class cBoundSphere implementation------------*/


/********************************************************************
*	class cBounderBox implementation								*
********************************************************************/

//	check if a point is inside the bounding box
bool cBoundBox::CheckCollision(point3d& p)
{
	//	make the test..
	//	if any of the six conditions is not true...it will return false
	if(p.x>=m_Position.x-m_HalfDimensions.x)
		if(p.x<=m_Position.x+m_HalfDimensions.x)
			if(p.y>=m_Position.y-m_HalfDimensions.y)
				if(p.y<=m_Position.y+m_HalfDimensions.y)
					if(p.z>=m_Position.z-m_HalfDimensions.z)
						if(p.z<=m_Position.z+m_HalfDimensions.z)
							return true;
	return false;
}

//	check if the bounding box is below a certain point ( the cursor position)
bool	cBoundBox::Below(point3d& CursorPos)
{
	//	make the test..
	//	if any of the six conditions is not true...it will return false
	if(CursorPos.x>=m_Position.x-m_HalfDimensions.x)
		if(CursorPos.x<=m_Position.x+m_HalfDimensions.x)
			if(CursorPos.y>=m_Position.y-m_HalfDimensions.y)
				if(CursorPos.y<=m_Position.y+m_HalfDimensions.y)
					return true;
	return false;
}

//	scale the box
void	cBoundBox::Scale(point3d& Scale)
{
	m_HalfDimensions.x*=Scale.x;
	m_HalfDimensions.y*=Scale.y;
	m_HalfDimensions.z*=Scale.z;
}
/*-------------end of class cBoundBox implementation---------------*/



/********************************************************************
*		class rigid implementation									*
*	By Shady El Mously												*
*	this class represents the rigid body, based on hmodel			*
********************************************************************/

//	this is an initizlization function
//set mass  one and zero out velocity and acelration
void cRigidBody::Init(void)
{
	m_Mass					=	1.0f				;
	m_RotAcc				=	point3d::Zero		;
	m_TransAcc				=	point3d::Zero		;
	m_TransVel				=	point3d::Zero		;
	m_RotVel				=	point3d::Zero		;
	m_PrevLoc				=	point3d::Zero		;
	m_PrevTransVel			=	point3d::Zero		;
	m_Bounder				=	new cBounder(this)	;
	m_BBox					=	point3d(0,0,0)		;
}

//	this should update the object poisition and velocty
//	the handling is now still frame dependant not time dependant
void cRigidBody::UpDate(float dt)
{
	m_PrevLoc		=	m_Loc		;
	m_PrevTransVel	=	m_TransVel	;

	m_WorldMatrix.rotate	(m_RotVel.x*dt,m_RotVel.y*dt,m_RotVel.z*dt);
	m_WorldMatrix.translate	(m_TransVel*dt/100.0f);

	m_RotVel	+=m_RotAcc*dt/100.0f;
	m_TransVel	+=m_TransAcc*dt/100.0f;
}


//	this should handle collision
//	and reflect the velocity of 2 objects on the plane of collision
//	this is a heavy function...call only when necessary
void cRigidBody::HandleCollision(cRigidBody& Obj1)
{
	cRigidBody& Obj2=*(this);

	//	method 2:	Exact method
	point3d V1x,V2x;

	point3d&	V1=Obj1.m_TransVel;
	point3d&	V2=Obj2.m_TransVel;

	float M1=Obj1.m_Mass;
	float M2=Obj2.m_Mass;

	//	a-	find X-Axis
	point3d x_axis;//=Obj2.m_loc-Obj1.m_loc;
	x_axis.normalize();

	//	b-	Find Projections
	point3d U1x		=	x_axis*(x_axis*Obj1.m_TransVel);
	point3d U1y		=	Obj1.m_TransVel-U1x;

	point3d U2x		=	-x_axis*(-x_axis*Obj2.m_TransVel);
	point3d U2y		=	Obj2.m_TransVel-U2x;
	
	//	c-	Find New Velocities;
	V1x	=	((U1x*M1)+(U2x*M2)-(U1x-U2x)*M2)/(M1+M2);
	V2x	=	((U1x*M1)+(U2x*M2)-(U2x-U1x)*M1)/(M1+M2);

	//	d-	Find Final Velocities
	V1	=	V1x+U1y;
	V2	=	V2x+U2y;
}



//	this will reflect the velocity component
//	on a plane defined by the noirmal vector sent here..
void cRigidBody::Reflect(const point3d& Norm,float Loss)
{
	//m_Loc		=	m_PrevLoc				;
	m_TransVel	=	m_PrevTransVel	-	Loss*m_PrevTransVel	;
	plane3d colplane(Norm,0)				;
	m_TransVel=colplane.reflect(m_TransVel)	;
}

//	this checks if the bounding sphere has hit
//	a plane return true if the plane is touched
bool cRigidBody::HitPlane(plane3d& Plane)
{
	bsphere3d bball;
	bball.m_loc=m_Loc;
	bball.m_radius=m_Radius;
	return Plane.testsphere(bball)==sphereCoplanar ? true : false;
}


//	check uysing bounder spheres
bool	cRigidBody::SphereCheck(cRigidBody& Obj1)
{
	float dist=(m_Loc-Obj1.m_Loc).mag();

	if(dist<(m_Radius+Obj1.m_Radius))
		return true;
	return false;
}

//	check using bounding boxes
bool cRigidBody::BoxCheck(cRigidBody& other)
{
	point3d& object=other.m_Loc;

	point3d boxmin=-m_BBox/2+m_Loc;
	point3d boxmax=m_BBox/2+m_Loc;

	if((object.x>=boxmin.x&&object.x<=boxmax.x)&&
		(object.y>=boxmin.y&&object.y<=boxmax.y)&&
		(object.z>=boxmin.z&&object.z<=boxmax.z))
		return true;

	return false;
}

//	scaling functions..also scale the bounding box and sphere
void cRigidBody::Scale(float sx,float sy,float sz)
{
	c3DObject::Scale(sx,sy,sz);
	m_Radius*=sqrt(sx*sx+sy*sy+sz*sz);
	m_BBox.x*=sx;
	m_BBox.y*=sy;
	m_BBox.z*=sz;
}

void cRigidBody::Scale(float s)
{
	c3DObject::Scale(s);
	m_Radius*=s;
}

//	this will draw the bouding sphere of the object
void cRigidBody::ShowBounder()
{
	glPushMatrix();
	glMultMatrixf(m_WorldMatrix.d);
	auxWireSphere(m_Radius);
	glPopMatrix();
}

//	make the object a phere
void cRigidBody::CreateSphere(float radius,int numSphereRings,int numSphereSegments)
{
	c3DObject::CreateSphere(radius,numSphereRings,numSphereSegments);
	SAFE_DELETE(m_Bounder);
	m_Radius=radius;
	m_BBox.x=radius*2;
	m_BBox.y=radius*2;
	m_BBox.z=radius*2;
	m_Bounder=	new cBoundSphere(this,radius);
}

//	this will generate an indexed box mesh
void cRigidBody::CreateBox(float x,float y,float z)
{
	c3DObject::CreateBox(x,y,z);
	SAFE_DELETE(m_Bounder);
	m_Radius=0.25f*sqrt(x*x+y*y+z*z);
	m_BBox.x=x;
	m_BBox.y=y;
	m_BBox.z=z;
	m_Bounder=	new cBoundBox(this,point3d(x,y,z)/2.0f);
}


//	creating a h3d object
void cRigidBody::CreateObject(char* filename)
{
	c3DComplexObject::CreateObject(filename);
	SAFE_DELETE(m_Bounder);
	SAFE_DELETE(m_Bounder);
	m_Radius=0.707f;
	m_BBox.x=1.4f;
	m_BBox.y=1.4f;
	m_BBox.z=1.4f;
	m_Bounder=	new cBoundSphere(this,0.707f);
}


//	this will generate a disk
void cRigidBody::CreateDisk(float InRadius,float OutRadius)
{
	c3DObject::CreateDisk(InRadius,OutRadius);
	SAFE_DELETE(m_Bounder);
	m_BBox.x=OutRadius*2;
	m_BBox.y=OutRadius*2;
	m_BBox.z=0;
	m_Radius	=	OutRadius;
	m_Bounder	=	new	cBoundSphere(this,OutRadius);
}

//	create a cylinder
void cRigidBody::CreateCylinder(float baseradius,float topradius, float height)
{
	c3DComplexObject::CreateCylinder(baseradius,topradius,height);
	SAFE_DELETE(m_Bounder);
	m_Radius=0.25f*sqrt(baseradius*baseradius+topradius*topradius+height*height);
	float Y=(baseradius > topradius ? baseradius:topradius)*2;
	m_BBox.x=Y;
	m_BBox.z=Y;
	m_BBox.y=height;
	m_Bounder	=	new cBoundBox(this,point3d(Y,height/2,Y));
}

//	makes a torus
void cRigidBody::CreateTorus(float R1,float R2)
{
	c3DComplexObject::CreateTorus(R1,R2);
	SAFE_DELETE(m_Bounder);
	m_BBox.x=R2*2;
	m_BBox.y=R2*2;
	m_BBox.z=0;
	m_Radius=	R2;
	m_Bounder	=	new	cBoundSphere(this,R2);
}


//	create a text object
void cRigidBody::CreateText(char * String)
{
	c3DComplexObject::CreateText(String);
	SAFE_DELETE(m_Bounder);
	m_Radius=strlen(String)*0.25f;
	m_BBox.x=strlen(String)*0.5f;
	m_BBox.y=1;
	m_BBox.z=1;
	m_Bounder=	new cBoundBox(this,point3d(strlen(String)*0.25f,0.4f,0.5f));
}
/*------------------end of class rigid implementation-----------------*/

