#ifndef _PHYSICS_H
#define _PHYSICS_H
/********************************************************************
*	physics.h														*
*	part of the flulib class library								*
*	this file is the header for the classes simulating all			*
*	stuff realting to physics modeling								*
*	like grvity, force, momentum, and velocity, acceleration		*
*	classes from here can be used with classes from 3d types		*
*	to make real cool classes of object interacting					*
*	all code here is based on the math layer						*
********************************************************************/

#include "3dobjects.h"		//	here we take 3d models and give them physical properties

class cRigidBody;



/********************************************************************
*	class cBounder													*
*	abstracts the bounding of an object, can be used to check		*
*	the collision of objects this is abse class for bouding spheres	*
*	and bounding boxes, maybe ellpsoids later						*
********************************************************************/
class cBounder
{
public:

	//	this should point to the object location(center of it..in its world matrix)
	point3d&		m_Position		;

	//	always construst from an existing rigid body
	cBounder(cRigidBody* pBody);

	//	3d and 2d checks
	virtual bool	CheckCollision(point3d&){return false;}
	virtual bool	Below(point3d&){return false;}
	virtual	void	Scale(point3d& Scale){}
};
/*-------------------end of class cBounder declaration--------------*/


/********************************************************************
*	class cBoundSphere												*
*	absrarct a boudning box, inherited from the cBounder			*
********************************************************************/
class cBoundSphere : public cBounder
{
protected:
	
	float		m_Radius		;

public:

	cBoundSphere(cRigidBody* pBody,float radius):cBounder(pBody),m_Radius(radius)
	{		}

	//	check the existence of a vertex ins ide the bounding sphere
	bool CheckCollision(point3d& p);

	//	same text but only using x,y coordiantes, this is used for detecting cursor
	bool	Below(point3d& CursorPos);

	//	scale the sphere
	void	Scale(point3d& Scale);

};
/*---------------end of class cBoundSphere declaration-------------*/


/********************************************************************
*	class cBoundBox, abstracts a bounding box for collision tests	*
********************************************************************/
class cBoundBox : public cBounder
{
public :

	//	use half dimensions to reduce calculations
	point3d	m_HalfDimensions;

	//	always construst sedning the box dimensions
	cBoundBox(cRigidBody* pBody,point3d Dimensions):cBounder(pBody),m_HalfDimensions(Dimensions)
	{}

	//	check if a point is inside the bounding box
	bool CheckCollision(point3d& p);
	
	//	check if the bounding box is below a certain point ( the cursor position)
	bool	Below(point3d& CursorPos);

	//	scale the box
	void	Scale(point3d& Scale);

};
/*--------------end of class cBoundBox declaration-----------------*/


/********************************************************************
*	class rigid	declaration											*
*	this simulates a rigid body motion in 3d space					*
*	this class is inherited from bsphere3d, and adds physical		*
*	behiavior of the object,, from this class and the d model		*
*	classes, the motion can be viewed								*
*	also, a new additional functon for generting strieded meshes	*
*	and a function (DrawStried to draw such meshes					*
********************************************************************/
				
class cRigidBody : public c3DComplexObject
{
protected:

	
	bool		m_bInCollision	;		//	true during collisions
	float		m_Radius;				//	radius of bounding sphere
	point3d		m_BBox;					//	the bounding box dimensions


	//	this is an initizlization function
	//	set mass  one and zero out velocity and acelration
	void	Init(void);

public:

	

	//	this is the bounder used for collision.. can be sphere or box ( until now )
	cBounder	*	m_Bounder	;

	point3d			m_TransVel;		//	translational velocity
	point3d			m_TransAcc;		//	translational acceleration
	point3d			m_RotAcc;		//	rotational acceleration
	point3d			m_RotVel;		//	rotational velocity
	float			m_Mass;			//	and this is the object mass

	//	previous data...
	point3d			m_PrevLoc;			//	previous location
	point3d			m_PrevTransVel;		//	translational velocity

	

	//	this initializes using an o3d or an oe3t file
	//	and also zeros the values and mass =1.0 kg
	cRigidBody() : m_Radius(0)
	{
		Init();
	}


	//	destructr should deallocate the bounder
	~cRigidBody()
	{
		SAFE_DELETE(m_Bounder);
	}

	//	this should update the object poisition and velocty
	//	the handling is now still frame dependant not time dependant
	void UpDate(float);

	//	this checks if the bounding sphere has hit
	//	a plane return true if the plane is touched
	bool HitPlane(plane3d& Plane);

	//	collision test using bounding sphere
	bool	SphereCheck(cRigidBody& Other);
	
	//	collision test using bounding box
	bool	BoxCheck(cRigidBody& Other);

	//	this will reflect the velocity component
	//	on a plane defined by the noirmal vector sent here..
	void	Reflect(const point3d& Norm,float Loss=0.0f);
	//	this should handle collision
	//	and reflect the velocity of 2 objects on the plane of collision
	void	HandleCollision(cRigidBody& Obj1);
	//	this will draw the bouding sphere of the object
	void	ShowBounder();

	//	some old functions redefined here..virtual in the super class

	//	scale calls both scaling functions for the sphere and for the hmodel
	void	Scale (float factor);
	//	scale calls both scaling functions for the sphere and for the hmodel
	void	Scale (float sx,float sy,float sz);
	
	// this will set the velocity

	// make the object a phere
	void CreateSphere(float radius,int numSphereRings=30,int numSphereSegments=30);

	// this will generate an indexed box mesh
	void CreateBox(float x, float y, float z);

	// creating an object
	void CreateObject(char* filename);

	// this will generate a disk
	void CreateDisk(float InRadius,float OutRadius);

	// create a cylinder
	void CreateCylinder(float baseradius,float topradius, float height);

	// make a Torus
	void CreateTorus(float R1,float R2);

	// make a 3d text object
	void CreateText(char * String);
};


/*------------------end of class rigid declaration-------------*/



#endif //_PHYSICS_H