#include "scene.h"
#include "input.h"

/****************************************************************
*	scene.cpp													*
*	by shady el mously											*
*	implementation of the scene class							*
*	class c3DScene implementation								*
****************************************************************/

//	this will render a ingle frame of the scene
int c3DScene::Render()
{
	int i;

	//	if the scene is not loaded, load it
	if(!m_bIsLoaded)		m_SceneData->Create(this);

	//	in case there is a view controller.. use it to adjust the camera
	if(m_SceneData->m_pViewer)
	{
		m_SceneData->m_pViewer->GetData();
		m_SceneData->m_HeadResponse.Apply(*(point3d*)&m_SceneData->m_pViewer->m_Yaw);
		(*(point3d*)&m_SceneData->m_Camera.m_Yaw)=m_SceneData->m_HeadResponse.Output();
		
		if(m_SceneData->m_bKeyboardControl)
		{
			point3d KeysData(0,0,0);

			if(KEYDOWN(VK_UP))
				KeysData.z=m_SceneData->m_Camera.m_Speed;
			if(KEYDOWN(VK_DOWN))
				KeysData.z=-m_SceneData->m_Camera.m_Speed;
			if(KEYDOWN(VK_LEFT))
				KeysData.x=-m_SceneData->m_Camera.m_Speed;
			if(KEYDOWN(VK_RIGHT))
				KeysData.x=m_SceneData->m_Camera.m_Speed;

			m_SceneData->m_KeysResponse.Apply(KeysData);

			KeysData=m_SceneData->m_KeysResponse.Output();

			m_SceneData->m_Camera.TranslateLocal(KeysData.x,KeysData.y,KeysData.z);
		}
		m_SceneData->m_Camera.ComputeInfo();
	}

	//	activate all light in the scene
	for(i=0;i<m_NumLights;i++)
		m_pLights[i]->Activate();
	
	//	Render 3D Stuff
	//	loop on all normal objects and draw them
	for(i=0;i<m_NumObjects;i++)
	{
		m_pObjects[i]->Draw();
	}
	//	loop on all rigid bodies and draw and update them
	for(i=0;i<m_NumRigidBodies;i++)
	{
		m_pRigidBodies[i]->UpDate(m_DeltaTime);
	}

	//	if there is a menu.. process it first and draw it
	if(m_SceneData->m_pMenu)
		if(0==HandleMenu())
			return 0;
	
	//	preparing to draw the particle systems
	glEnable(GL_BLEND);					//	enable alpha
	glDisable(GL_DEPTH_TEST);			//	disable depth buffer test
	glDisable(GL_LIGHTING);				//	lighting
	
	//	update all particle systems and render them
	for(i=0;i<m_NumParticles;i++)
	{
		m_pParticles[i]->Update(m_DeltaTime);
		m_pParticles[i]->Render();
	}
	glDisable(GL_BLEND);				//	disable alpha
	glEnable(GL_DEPTH_TEST);			//	enable depth buffer test
	glEnable(GL_LIGHTING);
	


	//	Render some 2D text or stuff...
	Graphics()->InitDrawing2D();
	//	show the frame rate if required
	if(m_SceneData->m_bShowFps)				Graphics()->WriteFPS();
	//	show the marquee if required
	if(m_SceneData->m_pMarquee)
	{
		m_SceneData->m_pMarquee->Draw();
		if(m_SceneData->m_pMarquee->Done())
			return 0;
	}
	Graphics()->EndDrawing2D();

	return 1;
}

//	update objects position in the scene
void c3DScene::Update()
{
	int i;	//	for looping

	//	go one step in all paths
	for(i=0;i<m_NumPaths;i++)
			m_pPath[i]->Next();

	//	apply all paths links
	for(i=0;i<m_NumPathLinks;i++)
		*m_pPathLink[i].m_pPoint=m_pPathLink[i].m_pPath->GetCurr()*m_pPathLink[i].m_Scale;

	//	apply all point links
	for(i=0;i<m_NumPointLinks;i++)
		*m_pPointLink[i].m_pDestPoint=*m_pPointLink[i].m_pSrcPoint*m_pPointLink[i].m_Scale;
}

//	this will play the scene
//	this is a very general rendering loop...that will draw the custion scene
void c3DScene::Play()
{

	int i;	//	for looping


	if(!m_bIsLoaded)		m_SceneData->Create(this);


	//	simulation loops starts
	while(!Application()->ProcessMessages())
	{
		//	when its time to draw a frame
		if(FrameController()->IsReady())
		{
			//	read input
			Input()->Catch();

			if(KEYDOWN(VK_SPACE)&&Application()->m_KeyPressed)
			{
				Application()->m_KeyPressed=false;
				break;
			}
			if(KEYUP(VK_SPACE)&&!Application()->m_KeyPressed)
			{
				Application()->m_KeyPressed=true;
			}
	
			//	Process Physics and/or AI
			Update();



			for(i=0;i<m_NumBounded;i++)
			{
			//	if the is a cursor restrict it inside the scene world (maxpoint and min point)
			if(m_pBounded[i]->m_Bounder->m_Position.x<m_SceneData->m_MinPoint.x)
				{
					m_pBounded[i]->m_Bounder->m_Position.x=m_SceneData->m_MinPoint.x;
					m_pBounded[i]->Reflect(point3d(1,0,0));
				}
				if(m_pBounded[i]->m_Bounder->m_Position.x>m_SceneData->m_MaxPoint.x)
				{
					m_pBounded[i]->m_Bounder->m_Position.x=m_SceneData->m_MaxPoint.x;
					m_pBounded[i]->Reflect(point3d(-1,0,0));
				}
				if(m_pBounded[i]->m_Bounder->m_Position.y<m_SceneData->m_MinPoint.y)
				{
					m_pBounded[i]->m_Bounder->m_Position.y=m_SceneData->m_MinPoint.y;
					m_pBounded[i]->Reflect(point3d(0,1,0));
				}
				if(m_pBounded[i]->m_Bounder->m_Position.y>m_SceneData->m_MaxPoint.y)
				{
					m_pBounded[i]->m_Bounder->m_Position.y=m_SceneData->m_MaxPoint.y;
					m_pBounded[i]->Reflect(point3d(0,-1,0));
				}
				if(m_pBounded[i]->m_Bounder->m_Position.z<m_SceneData->m_MinPoint.z)
					m_pBounded[i]->Reflect(point3d(0,1,0));

				if(m_pBounded[i]->m_Bounder->m_Position.y>m_SceneData->m_MaxPoint.y)
					m_pBounded[i]->Reflect(point3d(0,-1,0));
			}

			//	Clear last frame
			glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

			//	update the camera
			m_SceneData->m_Camera.Activate();

			//	render everything and break if necessary
			if(0==Render())
				break;

			if(Application()->m_IsDead)
				break;

			//	flush buffers
			glFlush();

			//	Flip the frame 
			Graphics()->Flip();
			FrameController()->Update();
		}
	}

	//	unload everything
	m_SceneData->Kill();
	//	now it is not loaded...everything is wiped from memory..hola !
	m_bIsLoaded	=	false;

	//	now if there is a next scene...play it
	if(strlen(m_SceneData->m_NextSceneFilename))
	{
		c3DScene NextScene(m_SceneData->m_NextSceneFilename);
		NextScene.Play();
	}
}


//	handling the 3d menu
int c3DScene::HandleMenu()
{
	//	loop an all buttons and run their scene if any was clicked
	//	to run their scene..deallocate this scene and allocate it when back
	for(int i=0;i<m_SceneData->m_pMenu->m_SceneButtons.size();i++)
	{
		if(m_SceneData->m_pMenu->m_SceneButtons[i]->IsClicked())
		{
			//	get the file name to open before deallocating..
			string SceneFileName=m_SceneData->m_pMenu->m_SceneNames[i];
			//	kill this scene
			m_SceneData->Kill();
			//	now it is not loaded...everything is wiped from memory..hola !
			m_bIsLoaded	=	false;
			//	make the child scene
			c3DScene ChildScene((char*)SceneFileName.c_str());
			//	and play it
			ChildScene.Play();
			//	when back recreate this scene again
			m_SceneData->Create(this);
			//	and re-run the marquee here
			if(m_SceneData->m_pMarquee)
				m_SceneData->m_pMarquee->Start();
		}
	}
	//	if there is an exit button and it is click...leave this loop
	if(m_SceneData->m_pMenu->m_pExitButton)
		if(m_SceneData->m_pMenu->m_pExitButton->IsClicked())
			return 0;
	//	draw the menu
	m_SceneData->m_pMenu->Draw();
	return 1;
}

/*----------------end of class c3DScene implementation----------*/