#ifndef _SCENE_H
#define _SCENE_H
#include "scenedata.h"
#include "framecontroller.h"

/********************************************************************
*	class c3DScene													*
*	used for abstarcrting 3d scene described in text files			*
*	this class have things of every type in the library				*
*	and can be used to describe world and render it					*
*	use Render() to render thw world described in the sript			*
*	or use Play to playback the entire script description			*
********************************************************************/
class c3DScene
{
private:

	cSceneData*	m_SceneData;	//	the scene data will be stored here
	float&		m_DeltaTime;	//	timing information


public:

	//	pointers needed while rendering
	cParticleSystem**	m_pParticles	;//	pointer to the particle systems
	c3DObject**			m_pObjects		;//	use a pointer to loop on the objects
	cRigidBody**		m_pBounded		;//	pointer to bounded objects
	
	cRigidBody**		m_pRigidBodies	;//	use a pointer to loop on the rigid bodies
	cPath**				m_pPath			;//	pointer to paths
	sPathLink*			m_pPathLink		;//	pointer to path links
	sPointLink*			m_pPointLink	;//	pointer to point links
	sLight**			m_pLights		;//	pointer to lights

	int					m_NumParticles		;//	number of particle systems
	int					m_NumObjects		;//	number of objects
	int					m_NumBounded		;//	number of bounded objects
	int					m_NumRigidBodies	;//	number of rigid bodies
	int					m_NumPaths			;//	number of paths
	int					m_NumPathLinks		;//	number of path links
	int					m_NumPointLinks		;//	number of point links
	int					m_NumLights			;//	number of lights

	cCamera*			m_pCamera			;//	the scene camera

	bool				m_bIsLoaded			;//	true when the scene is actually loaded



	//	contstruct always with a file name
	c3DScene(char* filename): m_DeltaTime(FrameController()->m_DeltaTime)
	{
		m_SceneData	=	new cSceneData(filename);
		m_bIsLoaded	=	false;
	}

	//	destrctor get rid of everything
	virtual ~c3DScene()
	{
		SAFE_DELETE(m_SceneData);
	}

	//	renders a single frame only..return 0 to break
	int Render();

	//	updates the objects and stuff in the scene
	void Update();

	//	play out the scene, will exit on pressing the space bar or escape
	void Play();

	//	handling the 3d menu, return 0 for exit
	int HandleMenu();
};

/*-----------------end of class c3DScene declaration----------------*/
#endif // _SCENE_H