#ifndef _SCENELOAD_H
#define _SCENELOAD_H

/********************************************************************
*	sceneload.h														*
*	By Shady El Mously												*
*	this is the header file for the class responsible on loading	*
*	a scene from a text file into its scene object					*
*	i made this class during to the hugeness of the scene class		*
*	so id ecided tos plit it into 2 parts, one for loading only		*
*	and the other will contain the scene itself						*
********************************************************************/
#include "scenedata.h"
/********************************************************************
*	class cSceneLoader	declaration									*
********************************************************************/


class cSceneLoader
{
private :


	//	we have a pointer to the scene we load into..
	cSceneData* m_pScene;


	FILE*				m_Fptr			;	//	file pointer to read from


	//	here i will store the names of everything, static becuase only needed once
	vector<string>		m_ObjectsNames		;	//	the names of all object in the scene, in the same order they are in their vector
	vector<string>		m_PointsNames		;	//	the point3d names..
	vector<string>		m_PathsNames		;	//	names of the paths
	vector<string>		m_PathLinksNames	;	//	pathlinks names
	vector<string>		m_LightsNames		;	//	lights names
	vector<string>		m_PointLinksNames	;	//	pointlinks names
	vector<string>		m_ParticlesNames	;	//	particle systems names


	vector<cString>&	m_LineWords		;	//	alias for cString::arg_v

	//	read the scene header, make sure it is correct
	//	return 0 on failure
	int ReadScene();

	//	this will read all the file data
	int ReadData();

	//	read a single object description data
	int ReadObject();

	//	read a camera description
	int ReadCamera();

	//	read a light
	int ReadLight();

	//	read a path
	int	ReadPath();

	//	read a link between a path and a point3d
	int	ReadPathLink();

	//	read a link between a 2 point3dz
	int	ReadPointLink();

	//	read a particle system
	int	ReadParticleSystem();

	//	read some data about the text marquee displayed
	int	ReadMarquee();

	//	add a new variable to be controlled
	int	ReadPoint3d();

	//	add a fog
	int	ReadFog();

	//	return true if the line is a comment
	bool IsComment();

	//	return true if this is new section
	bool NewSection();

	//	returns the index of the object name, return -1 if no such name exists
	int	GetObjectID(char* );

	//	returns the index of the point3d name, return -1 if no such name exists
	int	GetPointID(char* );

	//	returns the index of the path name sent, -1 means not found
	int	GetPathID(char* );

	//	returns the index of the pathlink name sent, -1 means not found
	int	GetPathLinkID(char* );

	//	returns the index of the light name sent, -1 means not found
	int	GetLightID(char* );

	//	returns the index of the particle system name sent, -1 means not found
	int	GetParticleSystemID(char* );


	//	clear the names when done reading the file
	void ClearNames();

	//	this will prepare a line of text, skipping any empty lines
	void PrepareLine();




public:


	//	only construct using the address of an existing scene
	cSceneLoader(cSceneData* pScene):m_pScene(pScene),m_LineWords(cString::arg_v)
	{
		m_Fptr		=	NULL			;
	}


	//	this will load the scene from the file
	int Load();


	//	use those to encode/decode files
	static void Encode(char* infilename,char* outfilename);
	static void Decode(char* infilename,char* outfilename);


};
/*----------------end of cSceneLoader declaration------------------*/
#endif //_SCENELOAD_H