#ifndef _TEXT_H
#define _TEXT_H
/****************************************************************
*	Text and imput/output section								*
*	and the command line interpreter							*
****************************************************************/

#include "base.h"		//	for singleton and tthread classes
#include <map>			//	for mapping between keyboard and ascii
using std::map;

/****************************************************************
*	Class cTextSubSystem										*
*	This class encapsulates all the functions					*
****************************************************************/

class cTextSubSystem : public singleton<cTextSubSystem>
{
private:

	int&	m_CurLine			;	//	current we are typing to
	int&	m_CurCharPos		;	//	current character position in the current line 
	int		m_NumLines			;	//	number of lines
	int		m_NumCharsPerLine	;	//	number os max charcaters per line
	char**	m_Buffer			;	//	the pointer to screen text buffer, will be taen from Graphics()
	char	m_LastCharacter		;	//	lasttyped character
	DWORD	m_CurTime			;	//	current time
	bool	m_DelayOnAll		;	//	delay on all charcaters flags(false recommended)


	void BuildKeysMap(void);		//	will build the map at startup between keyboard and Ascii Charcaters

public:

	map<DWORD,char>	m_VKtoAsciiMap;	//	map between virtual keys and ascii characters

	//	constructor, a graphics layer pointer  of an existing  one must be sent
	cTextSubSystem();

	//	destructor just reports the success of shutting down the text subsystem
	~cTextSubSystem()
	{
		Application()->m_ReportFile<<"Textlayer   SubSystem Successfully ShutDown   "<<endl;
	}

	//	Beautiful functions for string I/O
	char Getch(void);			//	used to get a single character
	void PutString(char * string,int x_pos=-1,int y_pos=-1);
	void GetString(char * string);
	void Clear();


	//	Cursor Control Functions..
	void	NewLine		(void)	;
	void	ScrollUp	(void)	;
	void	BackSpace	(void)	;
	void	SendChar	(char)	;
	char	GetKey		(void)	;

	//	return the ascii code of a touched keyboard key
	//	returns 0 if nothing was touched
	char HitKey();
};
//	public access to the text subsystem is from here..

inline cTextSubSystem	*&	Text()		{return cTextSubSystem::GetSingletonPtr();}
/*-------------------------------------------------------------*/
//				End of TEXTIO definition


/****************************************************************
*	class shell	definition										*
*	By Shady el Mously											*
*	part of the flulib class library							*
*	this class implement a shell (command line interpreter)		*
****************************************************************/
class cShell : public cThread
{
protected:
	
	cString		m_WelcomeMessage;		//	the welcome message

public:

	//	construct with a shell welcome message
	cShell(char* welcome="Welcome to Flulib Shell\n") : m_WelcomeMessage(welcome)
	{}

	//	sets the welcome message
	void SetWelcomeMessage(char* Str)
	{	m_WelcomeMessage	=	Str;	}

	//	this will start the shell as a normal function
	void Run(void);
	//	the default commands remain here
	bool HandleCommands();
	//	this should be defined in child classes to handle more commands
	virtual bool HandleMoreCommands(void){return false;}
};

/*-------------------------------------------------------------*/
//				End of class shell definition
#endif //_TEXT_H