#include "world.h"


/************************************************************
*	world.cpp												*
*	By Shady El Mously										*
*	implementation of the algorithms required to describe	*
*	an indoor environment and finding paths inside it		*
************************************************************/







/************************************************************
*	struct Room implementation								*
************************************************************/

//	draw the room 
void Room::Draw()
{
	point3d p3,p4;
	point3d norm;

	float d;
	glBegin(GL_QUADS);
	//	for each wall.. draw a line
	for(int i=0;i<m_Walls.size();i++)
	{
		p3=pWorld->m_Verts[m_Walls[i].a]+point3d(0,10,0);
		p4=pWorld->m_Verts[m_Walls[i].b]+point3d(0,10,0);


		d=point3d::dist(p3,p4)/10.0f;

		norm=(pWorld->m_Verts[m_Walls[i].a]-p3)^(p3-p4);

		glNormal3fv(norm.v);

		glTexCoord2f(0,0);
		glVertex3fv(p3.v);
		glTexCoord2f(d,0);
		glVertex3fv(p4.v);
		glTexCoord2f(d,1);
		glVertex3fv(pWorld->m_Verts[m_Walls[i].b].v);
		glTexCoord2f(0,1);
		glVertex3fv(pWorld->m_Verts[m_Walls[i].a].v);
	}
	glEnd();
}

//	copute the bounds of a room
void Room::ComputeBounds()
{
	m_MinPoint=point3d(9999,9999,9999);
	m_MaxPoint=point3d(-9999,-9999,-9999);

	
	//	loop on all doors of the room
	point3d TestPoint1;	//	points to test with
	point3d TestPoint2;	

	for(int i=0;i<m_Walls.size();i++)
	{
		//	get the two points of each wall
		TestPoint1=pWorld->m_Verts[m_Walls[i].a];
		TestPoint2=pWorld->m_Verts[m_Walls[i].b];

		//	check x
		if(TestPoint1.x>m_MaxPoint.x)
			m_MaxPoint.x=TestPoint1.x;
		if(TestPoint2.x>m_MaxPoint.x)
			m_MaxPoint.x=TestPoint2.x;

		if(TestPoint1.x<m_MinPoint.x)
			m_MinPoint.x=TestPoint1.x;
		if(TestPoint2.x<m_MinPoint.x)
			m_MinPoint.x=TestPoint2.x;

		//	check y
		if(TestPoint1.y>m_MaxPoint.y)
			m_MaxPoint.y=TestPoint1.y;
		if(TestPoint2.y>m_MaxPoint.y)
			m_MaxPoint.y=TestPoint2.y;

		if(TestPoint1.y<m_MinPoint.y)
			m_MinPoint.y=TestPoint1.y;
		if(TestPoint2.y<m_MinPoint.y)
			m_MinPoint.y=TestPoint2.y;

		//	check z
		if(TestPoint1.z>m_MaxPoint.z)
			m_MaxPoint.z=TestPoint1.z;
		if(TestPoint2.z>m_MaxPoint.z)
			m_MaxPoint.z=TestPoint2.z;

		if(TestPoint1.z<m_MinPoint.z)
			m_MinPoint.z=TestPoint1.z;
		if(TestPoint2.z<m_MinPoint.z)
			m_MinPoint.z=TestPoint2.z;
	}
}

//	test if a point is inside a room
//	first we check if inside the cube containing the room
bool Room::Contains(point3d& testPoint)
{
	//	first make the bounding box test
	if(
		testPoint.x<m_MaxPoint.x
		&&
		testPoint.x>m_MinPoint.x
		&&
		testPoint.z<m_MaxPoint.z
		&&
		testPoint.z>m_MinPoint.z
		)
	return true;
	else
		return false;
}
/*------------end of struct Room implementation------------*/



/************************************************************
*	struct World implementation								*
************************************************************/


//	world constructor
World::World(char* fileName)
{
	m_pGroundTexture	=	NULL;
	m_pWallTexture		=	NULL;

	ifstream infile;
	infile.open(fileName);

	int i,j;			//	loop index
	int NumVerts;
	int	NumDoors;
	int	NumWalls;
	int	NumRooms;
	point3d Vertex;
	point3d DoorCenter;
	Door	NewDoor;
	Room	NewRoom;
	Wall	NewWall;
	short WallIdx;	

	//	get number of vertices
	infile>>NumVerts;
	
	//	read the vertices line by line
	for(i=0;i<NumVerts;i++)
	{
		infile>>Vertex.x;
		infile>>Vertex.y;
		infile>>Vertex.z;
		//	add the vertex to the world
		m_Verts.push_back(Vertex);
	}

	//	read number of doors
	infile>>NumDoors;

	//	read door indices
	for(i=0;i<NumDoors;i++)
	{
		infile>>NewDoor.a;
		infile>>NewDoor.b;
		m_Doors.push_back(NewDoor);
	}


	//	read number of rooms
	infile>>NumRooms;

	//	for every room..
	for(i=0;i<NumRooms;i++)
	{
		//	setup the new room
		NewRoom.m_DoorsIndex.clear();
		NewRoom.m_Walls.clear();
		NewRoom.pWorld=this;

		//	read number walls in room
		infile>>NumWalls;
		
		//	for each wall
		for(j=0;j<NumWalls;j++)
		{
			infile>>NewWall.a;
			infile>>NewWall.b;

			//	add the wall to the room
			NewRoom.m_Walls.push_back(NewWall);
		}


		//	read number of doors
		infile>>NumDoors;

		//	read doors indices
		for(j=0;j<NumDoors;j++)
		{
			//	read the wall index
			infile>>WallIdx;
			NewRoom.m_DoorsIndex.push_back(WallIdx);
		}
		//	add the new room
		m_Rooms.push_back(NewRoom);
	}


	//	compute rooms bounds
	for(i=0;i<m_Rooms.size();i++)
		m_Rooms[i].ComputeBounds();
};

//	returns a pth from 2 points inside the world
bool World::FindPath(cPatternPath& NewPath,point3d src,point3d dst)
{
	//	index to the node of the room that has the points used in search
	int	SrcRoomNodeIndex=-1;
	int	DstRoomNodeIndex=-1;

	//	index of the source and destination room numbers
	int SrcRoomNum=-1;
	int DstRoomNum=-1;

	//	index for the points we will add to the nodes ( we will add 2 nodes to solve the algorithm )
	int	SrcIndex;
	int DstIndex;

	//	find the cells of the src and destination points
	for(int i=0;i<m_Rooms.size();i++)
	{
		if(m_Rooms[i].Contains(src))
		{
			SrcRoomNodeIndex=m_Rooms[i].m_NodeIndex;
			SrcRoomNum	=	i;
		}
		if(m_Rooms[i].Contains(dst))
		{
			DstRoomNodeIndex=m_Rooms[i].m_NodeIndex;
			DstRoomNum	=	i;
		}
	}

	//	if any is not in the world.. return
	if(SrcRoomNodeIndex==-1||DstRoomNodeIndex==-1)
		return false;


	//	if they are both the same
	//	make the path with the two points and return it
	if(SrcRoomNodeIndex==DstRoomNodeIndex)
	{
		NewPath.m_PointList.push_back(src);
		NewPath.m_PointList.push_back(dst);
		return true;
	}

	//	reaching here means that each point is in a separate room
	//	we should add the src and dst and two new nodes in the graph
	SrcIndex=m_Nodes.size();
	AddNode(src);

	DstIndex=m_Nodes.size();
	AddNode(dst);

	//	first link each new node to the node in the center of the cell it is contained in
	LinkNodes(SrcIndex,SrcRoomNodeIndex);
	LinkNodes(DstIndex,DstRoomNodeIndex);

	//	and link each of the to all the nodes in the walls of the rooms they are in

	//	for the source first
	for(i=0;i<m_Rooms[SrcRoomNum].m_DoorsIndex.size();i++)
	{
		LinkNodes(SrcIndex,m_Rooms[SrcRoomNum].m_DoorsIndex[i]);
	}

	//	then for the destination
	for(i=0;i<m_Rooms[DstRoomNum].m_DoorsIndex.size();i++)
	{
		LinkNodes(DstIndex,m_Rooms[DstRoomNum].m_DoorsIndex[i]);
	}

	//	now solve for the last two nodes 
	cSearchTree SolTree(*this);

	SrcIndex=m_Nodes.size()-1;
	DstIndex=m_Nodes.size()-2;

	//	here we get indices for the solution.. and fill the path
	vector<int> Solution;

	//	solve using A*
	Solution=SolTree.AStarPath(SrcIndex,DstIndex);

	//	build the path
	for(i=0;i<Solution.size();i++)
	{
		NewPath.m_PointList.push_back(m_Nodes[Solution[i]]);
	}

	//	get rid of last 2 node
	DropNode(SrcIndex);
	DropNode(DstIndex);

	return true;
}





//	build the skeleton graph of the world
void World::BuildGraph()
{
	int i,j;
	point3d Vertex;
	int Src,Dst;
	m_Nodes.clear();
	m_Links.clear();
	point3d DoorCenter;


	//	first create the nodes which are inside doors
	for(i=0;i<m_Doors.size();i++)
	{
		DoorCenter=(m_Verts[m_Doors[i].a]+m_Verts[m_Doors[i].b])/2;
		AddNode(DoorCenter);
	}
	
	//	loop on them all.. make a node in the rom center
	//	and make links from that node to all the node inside the doors of the room
	for(i=0;i<m_Rooms.size();i++)
	{
		//	get the center
		Vertex=m_Rooms[i].GetCenter();

		//	add it as a node
		AddNode(Vertex);

		//	we need the source index
		Src=m_Nodes.size()-1;

		//	this source should be stored inside the room itself
		m_Rooms[i].m_NodeIndex=Src;

		//	loop on all the doors of the room and create links
		for(j=0;j<m_Rooms[i].m_DoorsIndex.size();j++)
		{
			Dst=m_Rooms[i].m_DoorsIndex[j];
			LinkNodes(Dst,Src);
		}
	}
}
//	get the center of a room
point3d Room::GetCenter()
{
	//	loop on all walls vertices and add them
	//	then take the average and return it
	point3d Sum(0,0,0);
	int TotalNum=0;


	for(int i=0;i<m_Walls.size();i++)
	{
		Sum+=pWorld->m_Verts[m_Walls[i].a];
		Sum+=pWorld->m_Verts[m_Walls[i].b];
		TotalNum	+=	2;
	}
	Sum/=TotalNum;
	return Sum;
}



//	draw the world
void World::Draw()
{
	int i;

	if(m_pGroundTexture)
		m_pGroundTexture->Activate();

	

	glBegin(GL_QUADS);

	glNormal3f(0,1,0);
	glTexCoord2f(100,100);
	glVertex3f(100,0,-100);
	glTexCoord2f(100,0);
	glVertex3f(-100,0,-100);
	glTexCoord2f(0,100);
	glVertex3f(-100,0,100);
	glTexCoord2f(0,0);
	glVertex3f(100,0,100);


	glEnd();

	if(m_pWallTexture)
		m_pWallTexture->Activate();


	//	loop on all rooms and draw them 
	for(i=0;i<m_Rooms.size();i++)
	{
		m_Rooms[i].Draw();
	}

	//	loop on all doors and draw them
/*	glColor3f(1,0,0);

	glBegin(GL_LINES);
	for(i=0;i<m_Doors.size();i++)
	{
		glVertex3fv(m_Verts[m_Doors[i].a].v);
		glVertex3fv(m_Verts[m_Doors[i].b].v);
	}
	glEnd();

/*
	//	draw all nodes
	glColor3f(1,1,0);

	for(i=0;i<m_Nodes.size();i++)
	{
		glPushMatrix();
		glTranslatef(m_Nodes[i].x,m_Nodes[i].y,m_Nodes[i].z);
		auxSolidSphere(0.3f);
		glPopMatrix();
	}

	//	draw all links
	glColor3f(1,0,1);

	glBegin(GL_LINES);
	for(i=0;i<m_Links.size();i++)
	{
		for(int j=0;j<m_Links[i].size();j++)
		{
			if(m_Links[i][j]==true)
			{
				glVertex3fv(m_Nodes[i].v);
				glVertex3fv(m_Nodes[j].v);
			}
		}
	}
	glEnd();*/
}



//	returns true if in world
bool	World::Contains(point3d& p)
{
	//	loop on all rooms
	//	if inside anyroom.. return true immediately
	for(int i=0;i<m_Rooms.size();i++)
	{
		if(m_Rooms[i].Contains(p))
			return true;	
	}
	return false;
}


/*-------------end of struct world implementation---------*/