#include "library.h"
#include "graph.h"
#include "3dpaths.h"


#ifndef _WORLD_H
#define _WORLD_H

/************************************************************
*	world.h													*
*	By Shady el moulsy										*
*	header declaration for data structures describing		*
*	a complete world made of rooms and allowing searching	*
*	for paths inside it										*
************************************************************/


//	just say there will be a structure called world
struct World;

//	a wall need 2 indexes saying the vertices are whom
struct Wall
{	short a,b;	};

//	a door need 2 indexes saying the vertices are whom
struct Door
{	short a,b;	};


/************************************************************
*	struct Room												*
*	represnets a single room in a world						*
*	the single room has a set of walls and a set of indices	*
*	to the doors, a points to the world in which it is		*
*	and some bounds for checking the existence inside it	*
************************************************************/
//	a room have a set walls and doors
struct Room
{
	vector<Wall>	m_Walls;
	vector<short>	m_DoorsIndex;	//	indices to the doors of the rooms, doors are shared
	World*			pWorld;			//	a pointer to the world the room belongs to
	short			m_NodeIndex;	//	index to the node in the center of the room

	//	variable for checking
	point3d			m_MaxPoint;		//	max bounds of the room
	point3d			m_MinPoint;		//	min bounds of the room

	//	draw the room
	void Draw();

	//	get the center of a room
	point3d GetCenter();

	//	returns true if the 3d point sent is inside the room
	bool Contains(point3d& point);

	//	call this to find the max and min points
	void ComputeBounds();
};

/*-----------------end of struct Room declaration-----------*/


/************************************************************
*	struct World											*
*	represents the entire world of rooms					*
*	and allows finding paths inside this world				*
*	it has a road map representaion of the world			*
*	and the search is using A* algorithm					*
************************************************************/
struct World : public cGraph
{
	vector<Room>	m_Rooms;		//	rooms in the world, their walls are inside them
	vector<Door>	m_Doors;		//	doors (shared information
	vector<point3d>	m_Verts;		//	vertices used .. everything index inside this

	cTexture*		m_pGroundTexture;	//	texture used as ground
	cTexture*		m_pWallTexture;		//	texture used for walls


	//	construct with a text file
	World(char *FileName);

	//	destructor
	~World(){SAFE_DELETE(m_pGroundTexture);}


	//	draw the entire world..
	//	this function can be optimized greatly later using the portal rendering algorithm
	void Draw();


	//	this will build the graph
	void BuildGraph();


	//	send two points and get a linear path from one to the other
	bool FindPath(cPatternPath& Path,point3d src,point3d dst);

	//	returns true if in world
	bool	Contains(point3d& p);
};

/*--------------end of struct World declaration--------------*/



#endif //_WORLD_H